package com.msdw.tms.common.utils.trading_rules;

import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

/**
 * @ProjectName: tms
 * @Package: com.msdw.tms.common.utils.chuanda
 * @ClassName: algorithm
 * @Description: 川大交易规则算法
 * @Author: Maureen.Rong
 * @CreateDate: 2020/10/12 10:04
 * @UpdateDate: 2020/10/12 10:04
 * @Version: 1.0
 */
public class algorithm {

    /**
     *可交易数量:
     *1、多方情况下，可交易数量为多方合约持仓总量
     *2、空方情况下，可交易数量为空方合约持仓总量
     */

    //--------------------配置期货头寸START------------------------

    /**
     * 预冻结保证金: 预计冻结保证金 = 外汇期货合约价格 * 合约面值  * 保证金比例
     * 合约价格:    contractPrice
     * 合约面值:    contractFaceValue
     * 保证金比例:  bond
     */
    public static BigDecimal getPreFreezingMargin(BigDecimal contractPrice, BigDecimal contractFaceValue, String name) {

        try {
            //保证金比例
            BigDecimal bond = new BigDecimal(0);


            switch (name.substring(0, 8)) {
                case algorithm.AUDAndRMB:
                    bond = algorithm.AUDAndRMBBond;
                    break;
                case algorithm.EURAndRMB:
                    bond = algorithm.EURAndRMBBond;
                    break;
                case algorithm.JPYAndRMB:
                    bond = algorithm.JPYAndRMBBond;
                    break;
                case algorithm.RMBAndUSD:
                    bond = algorithm.RMBAndUSDBond;
                    break;
                case algorithm.USDAndRMB:
                    bond = algorithm.USDAndRMBBond;
                    break;

            }
            BigDecimal amount = contractPrice.multiply(contractFaceValue).multiply(bond);
            return amount;
        } catch (Exception e) {
            e.printStackTrace();
            return new BigDecimal(-1);
        }
    }


    /**
     * 保证金: 保证金 = 外汇期货合约价格 * 合约面值 * 委托数量 * 保证金比例
     * 合约价格:    contractPrice
     * 合约面值:    contractFaceValue
     * 保证金比例:  bond
     * 手数:       handCout
     */
    public static BigDecimal freezingMargin(BigDecimal contractPrice, BigDecimal contractFaceValue, BigDecimal handCout, String name) {

        try {
            //保证金比例
            BigDecimal bond = new BigDecimal(0.00);


            switch (name.substring(0, 8)) {
                case algorithm.AUDAndRMB:
                    bond = algorithm.AUDAndRMBBond;
                    break;
                case algorithm.EURAndRMB:
                    bond = algorithm.EURAndRMBBond;
                    break;
                case algorithm.JPYAndRMB:
                    bond = algorithm.JPYAndRMBBond;
                    break;
                case algorithm.RMBAndUSD:
                    bond = algorithm.RMBAndUSDBond;
                    break;
                case algorithm.USDAndRMB:
                    bond = algorithm.USDAndRMBBond;
                    break;

            }
            BigDecimal amount = contractPrice.multiply(contractFaceValue).multiply(handCout).multiply(bond);
            return amount;
        } catch (Exception e) {
            e.printStackTrace();
            return new BigDecimal(-1);
        }
    }

    /**
     * 预冻结手续费: 预冻结手续费 = 委托数量（手）* 费用标准
     * 委托数量: commissionQuantity
     * 费用标准: cost
     */
    public static BigDecimal getPreFreezingServiceCharge(BigDecimal commissionQuantity) {
        //暂定0.5%
        BigDecimal cost = new BigDecimal(0.005);
        BigDecimal amount = commissionQuantity.multiply(cost);
        return amount;
    }

    /**
     * 冻结资金:冻结资金=预计冻结保证金+预冻结手续费
     * 预计冻结保证金: preFreezingFunds
     * 预冻结手续费: preFreezingCharges
     */
    public static double getFreezingFunds(double preFreezingFunds, double preFreezingCharges) {
        return preFreezingFunds + preFreezingCharges;
    }

    /**
     * 需要资金:需要资金 = 委托价格 * 每手交易单位 * 委托数量(手数)
     * 委托价格: commissionPrice
     * 每手交易单位 : tradingUnit
     * 委托数量: commissionQuantity
     */
    public static BigDecimal getFundsRequired(BigDecimal commissionPrice, BigDecimal commissionQuantity, Integer tradingUnit) {
        return commissionPrice.multiply(commissionQuantity).multiply(BigDecimal.valueOf(tradingUnit));
    }

    //--------------------配置期货头寸 END------------------------

    //--------外汇期货每手价格 交易单位 START---------
    /**
     * 欧元兑人民币期货(EUR)
     */
    public static final int EURExchangeRMB = 50000;//欧元

    /**
     * 日元兑人民币期货(JPY)
     */
    public static final int JPYExchangeRMB = 6000000;//6000000 日元

    /**
     * 澳元兑人民币期货(AUD)
     */
    public static final int AUDExchangeRMB = 80000;//80000 澳元

    /**
     * 人民币兑美元期货(RMB)
     */
    public static final int RMBExchangeUSD = 30000;//30000 人民币
    /**
     * 美元兑人民币期货(USD)
     */
    public static final int USDExchangeRMB = 100000;//100000 美元

    //--------外汇期货每手价格 END---------


    //期货类型

    /**
     * 欧元兑人民币期货(EUR)
     */
    public static final String EURAndRMB = "欧元兑人民币期货";

    /**
     * 日元兑人民币期货(JPY)
     */
    public static final String JPYAndRMB = "日元兑人民币期货";

    /**
     * 澳元兑人民币期货(AUD)
     */
    public static final String AUDAndRMB = "澳元兑人民币期货";

    /**
     * 人民币兑美元期货(RMB)
     */

    public static final String RMBAndUSD = "人民币兑美元期货";
    /**
     * 美元兑人民币期货(USD)
     */
    public static final String USDAndRMB = "美元兑人民币期货";


    /**
     * 保证金设置
     */

    /**
     * 欧元兑人民币期货(EUR)
     */
    public static final BigDecimal EURAndRMBBond = new BigDecimal(0.023);
    /**
     * 日元兑人民币期货(JPY)
     */
    public static final BigDecimal JPYAndRMBBond = new BigDecimal(0.035);
    /**
     * 澳元兑人民币期货(AUD)
     */
    public static final BigDecimal AUDAndRMBBond = new BigDecimal(0.031);
    /**
     * 人民币兑美元期货(RMB)
     */
    public static final BigDecimal RMBAndUSDBond = new BigDecimal(0.17);
    /**
     * 美元兑人民币期货(USD)
     */
    public static final BigDecimal USDAndRMBBond = new BigDecimal(0.018);


    /**
     * 浮动盈亏=（当天结算价—昨日结算价）* 委托数量 * 合约面值(1份合约=一手，一手=？吨)
     * <p>
     * 当天结算价: todayPrice
     * 昨日结算价 : yesterdayPrice
     * 合约面值: contractFaceValue
     * 委托数量: commissionQuantity
     */
    public static BigDecimal getFloatingPL(BigDecimal todayPrice, BigDecimal yesterdayPrice, BigDecimal commissionQuantity, Integer contractFaceValue) {
        return todayPrice.subtract(yesterdayPrice).multiply(commissionQuantity).multiply(BigDecimal.valueOf(contractFaceValue));
    }

    /*public static void main(String[] args) {
        BigDecimal pl = getFloatingPL(new BigDecimal(6.826), new BigDecimal(6.84), new BigDecimal(1), 100000);
        System.out.println(pl);
    }*/

    /**
     * 盈亏=（平仓时的外汇汇率价格-下单时外汇汇率价格) * 委托数量 * 合约面值。
     * <p>
     * commissionQuantity：委托数量
     * contractFaceValue: 合约面值
     *
     * @return
     */
    /*public static BigDecimal getPL(BigDecimal price1, BigDecimal price2, BigDecimal commissionQuantity, BigDecimal contractFaceValue) {
        return price1.subtract(price2).multiply(commissionQuantity).multiply(contractFaceValue).setScale(2, BigDecimal.ROUND_DOWN);
    }*/

    /**
     * 外汇期货盈亏=（平仓时的外汇汇率价格-下单时外汇汇率价格) * 委托数量 * 合约面值。
     * <p>
     * commissionQuantity：委托数量
     * contractFaceValue: 合约面值
     *
     * @return
     */
    public static BigDecimal get_wh_pl(BigDecimal price1, BigDecimal price2, BigDecimal commissionQuantity, BigDecimal contractFaceValue) {
        // return price1.subtract(price2).multiply(commissionQuantity).multiply(contractFaceValue).setScale(2, BigDecimal.ROUND_DOWN);
        return price1.subtract(price2).multiply(commissionQuantity).multiply(contractFaceValue);
    }


    /**
     * 资产市值 = (开仓或者平仓) * 委托数量 * 合约面值(1份合约=一手，一手=？吨)
     * commissionQuantity：委托数量
     * contractFaceValue: 合约面值
     * 市值 = 外汇期货合约价格 * 委托数量 * 合约面值
     */
    public static BigDecimal getMarketValue(BigDecimal price, Integer commissionQuantity, Integer contractFaceValue) {
        return price.multiply(BigDecimal.valueOf(commissionQuantity)).multiply(BigDecimal.valueOf(contractFaceValue));
    }


    /**
     * 调仓操作
     * 加仓市值 = 外汇期货合约价格*（原持仓量 + 本次调仓量）* 合约面值，
     * price:外汇期货合约价格
     * old：原持仓量
     * news:本次调仓量
     * contractFaceValue: 合约面值
     */
    public static BigDecimal getMarketValueByAddition(BigDecimal price, BigDecimal old, BigDecimal news, Integer contractFaceValue) {
        BigDecimal a = price.multiply(new BigDecimal(contractFaceValue));
        BigDecimal b = old.add(news);
        return a.multiply(b);
    }


    /**
     * 调仓操作
     * 减仓市值 = 外汇期货合约价格*（原持仓量 - 本次调仓量）* 合约面值，
     * price:外汇期货合约价格
     * old：原持仓量
     * news:本次调仓量
     * contractFaceValue: 合约面值
     */
    public static BigDecimal getMarketValueBySubtraction(BigDecimal price, BigDecimal old, BigDecimal news, Integer contractFaceValue) {
        BigDecimal a = price.multiply(new BigDecimal(contractFaceValue));
        BigDecimal b = old.subtract(news);
        return a.multiply(b);
    }


    /**
     * 外币 = 汇率 * 合约面值
     *
     * @param price             汇率
     * @param contractFaceValue 合约面值
     * @return
     */
    public static BigDecimal getForeignCurrency(BigDecimal price, BigDecimal contractFaceValue) {
        return price.multiply(contractFaceValue);
    }

//----------------最后交易日------------------

    /**
     * 获取最后交易日
     * (1~4)
     * * 1.欧元兑人民币期货
     * * 2.日元兑人民币期货
     * * 3.澳元兑人民币期货
     * * 4.人民币兑美元期货
     * * <p>
     * * 5.美元兑人民币期货	最后结算日之前两个营业日
     *
     * @param name
     * @return 获取最后交易日
     */

    public static String getLastTradingDay(String name) {


        try {
            String getName = name.substring(0, 8);
            //年份
            String year = name.substring(8, 10);
            //月份
            String month = name.substring(10);

            //最后交易日为:合约月份的第3个星期3前两个营业日
            if (getName.equals(algorithm.AUDAndRMB) || getName.equals(algorithm.EURAndRMB) || getName.equals(algorithm.JPYAndRMB) || getName.equals(algorithm.RMBAndUSD)) {


                //获取某年某月的第3个星期天的星期三
                String reTime = TradingDay.getTheDate(Integer.valueOf("20" + year), Integer.valueOf(month), 3, 3);

                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");

                Date date = sdf.parse(reTime);

                String lastTime = getAddDate(date);

                return lastTime;
            } else if (getName.equals(algorithm.USDAndRMB)) {


                String times = "20" + year + "-" + month;
                //最后结算日之前两个营业日(本月的最后一个周五)
                String lastWeek = TradingDay.getLastWeek(times);
                return lastWeek;

            }
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return null;

    }


    //得到添加n天后的时间字符串
    public static String getAddDate(Date date) {
        try {
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");

            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);

            //减2天
            calendar.setTime(date);
            calendar.add(Calendar.DAY_OF_MONTH, -2);
            String T2 = format.format(calendar.getTime());
            return T2;//返回时间字符串

        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    /**
     * 根据获取的合约名称设置交易保证金
     *
     * @param name
     * @return
     */
    public static BigDecimal getBond(String name) {

        try {
            //保证金比例
            BigDecimal bond = new BigDecimal(0);

            switch (name.substring(0, 8)) {
                case algorithm.AUDAndRMB:
                    bond = algorithm.AUDAndRMBBond;
                    break;
                case algorithm.EURAndRMB:
                    bond = algorithm.EURAndRMBBond;
                    break;
                case algorithm.JPYAndRMB:
                    bond = algorithm.JPYAndRMBBond;
                    break;
                case algorithm.RMBAndUSD:
                    bond = algorithm.RMBAndUSDBond;
                    break;
                case algorithm.USDAndRMB:
                    bond = algorithm.USDAndRMBBond;
                    break;
            }
            return bond;
        } catch (Exception e) {
            e.printStackTrace();
            return new BigDecimal(-1);
        }
    }


}

