package com.msdw.tms.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.msdw.tms.common.utils.PageUtils;
import com.msdw.tms.common.utils.R;
import com.msdw.tms.common.utils.TokenUtil;
import com.msdw.tms.common.utils.trading_rules.algorithm;
import com.msdw.tms.entity.CdFuturesConfigureEntity;
import com.msdw.tms.entity.CdUserAssetsEntity;
import com.msdw.tms.entity.CdUserOptionAccountEntity;
import com.msdw.tms.entity.QhscGangjiaosuoEntity;
import com.msdw.tms.model.req.CdGetAccountInfoReq;
import com.msdw.tms.model.req.CdTransferRecordsReq;
import com.msdw.tms.model.req.CdTransferReq;
import com.msdw.tms.model.resp.CdAccountInfoResp;
import com.msdw.tms.model.resp.CdTransferOutResp;
import com.msdw.tms.model.resp.CdTransferRecordsResp;
import com.msdw.tms.service.*;
import com.msdw.tms.service.impl.CdFuturesConfigureServiceImpl;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.List;


/**
 * @描述：账户流水记录控制类
 * @作者: Rong
 * @日期: 2020-10-15
 */
@RestController
@RequestMapping("/chuanda/bankStatements")
@Api(value = "API - 川大:账户相关操作", tags = "川大子系统:账户相关操作")
public class CdBankStatementsController {

    @Autowired
    public CdBankStatementsService service;

    @Autowired
    public CdUserOptionAccountService optionAccountService;

    @Autowired
    public CdUserAssetsService assetsService;

    @Autowired
    public CdFuturesConfigureService configureService;

    @Autowired
    public QhscGangjiaosuoService gangjiaosuoService;
    @Autowired
    public RedisTemplate<String, String> template;

    /**
     * @描述: 银行/期货期权账户 转账操作
     * @作者: Rong
     * @日期: 2020-10-15
     **/
    @PostMapping("/transfer")
    @ApiOperation(value = "(银行/期货期权账户)转账操作")
    public R save(@RequestBody CdTransferReq req) {
        int userId = TokenUtil.getUserId();
        //查询银行账户余额
        CdUserAssetsEntity assets = assetsService.userAssetsByUerId(userId);
        //查询期权账户余额
        CdUserOptionAccountEntity account = optionAccountService.optionAccountByUerId(userId);

        switch (req.getTransferOut()) {
            case 1:
                if (assets.getBalance().compareTo(req.getAmountMoney()) == -1) {
                    return R.error("转账失败,您的银行账户可用余额不足以本次转账");
                }
                break;
            case 2:
                if (account.getBalance().compareTo(req.getAmountMoney()) == -1) {
                    return R.error("转账失败,您的期权期货账户可用余额不足以本次转账");
                }
                break;
        }

        if (service.modify(userId, req.getTransferOut(), req.getAmountMoney())) {
            return R.ok("转账成功");
        } else {
            return R.error();
        }

    }


    /**
     * @描述: 根据选择的转出账户查询可用资金
     * @入参:
     * @出参:
     * @作者: Rong
     * @日期: 2020-10-15
     **/
    @PostMapping(value = "/getCapital")
    @ApiOperation(value = "根据选择的账户查询资金", response = CdTransferOutResp.class)
    public R getCapital(@RequestParam("transferOut") @ApiParam(value = "账户类型(1.银行账户 2.期货账户)") Integer transferOut) {
        int userId = TokenUtil.getUserId();

        if (transferOut == 2) {
            //期权账户
            CdUserOptionAccountEntity account = optionAccountService.optionAccountByUerId(Long.valueOf(userId));
            return R.ok().put("data", account);
        } else if (transferOut == 1) {
            //银行账户
            CdUserAssetsEntity assets = assetsService.userAssetsByUerId(Long.valueOf(userId));
            return R.ok().put("data", assets);
        }
        return R.error();

    }


    /**
     * @描述: 转账记录
     * @入参:
     * @出参:
     * @作者: Rong
     * @日期: 2020-10-10
     **/
    @PostMapping(value = "/transferRecords")
    @ApiOperation(value = "转账记录", response = CdTransferRecordsResp.class)
    public R listByEntity(@RequestBody CdTransferRecordsReq req) {
        int userId = TokenUtil.getUserId();


        List<CdTransferRecordsResp> list = service.queryPageByRecode(String.valueOf(userId), req.getStartTime(), req.getEndTime());
        //总记录数
        int count = list.size();
        PageUtils pageUtils = new PageUtils(list, count, req.getPageSize(), req.getPageNum());

        return R.ok().put("page", pageUtils);

    }


    /**
     * 期货期权账户
     * 同时此处会返回待开仓的信息
     * 可用资金 = 当前账户可用资金
     *
     * @return
     */
    @PostMapping(value = "/getAccountInfo")
    @ApiOperation(value = "期货期权账户(此处会返回待开仓的信息)", response = CdAccountInfoResp.class)
    public R getAccountInfo(@RequestBody CdGetAccountInfoReq req) {

        int userId = TokenUtil.getUserId();
        R r = service.getAccountInfoReturnOpen(req, userId);
            /*//查询当前账户金额
            CdUserOptionAccountEntity entity = optionAccountService.optionAccountByUerId(userId);

            CdAccountInfoResp accountInfoResp = new CdAccountInfoResp();
            //可用资金
            accountInfoResp.setBalance(entity.getBalance());
            //冻结资金
            accountInfoResp.setForzenBalance(entity.getForzenBalance());
            //占用保证金
            accountInfoResp.setDepositBalance(entity.getDepositBalance());

            //每手交易单位(合约面值)
            Integer tradingUnit = 0;

            //查询当前用户的开仓信息
            QueryWrapper<CdFuturesConfigureEntity> queryWrapper = new QueryWrapper<CdFuturesConfigureEntity>();
            queryWrapper.eq("user_id", userId);

            BigDecimal float_pl = new BigDecimal(0);
            BigDecimal marketAssets = new BigDecimal(0);
            //查询当前用户持仓信息
            CdFuturesConfigureEntity configureEntity = configureService.getOne(queryWrapper);
            if (configureEntity != null) {

                //根据pid查询当前代码
                QhscGangjiaosuoEntity qhscGangjiaosuo = gangjiaosuoService.getById(configureEntity.getPid());

                //根据代码查询当天汇率价格
                QueryWrapper<QhscGangjiaosuoEntity> wrapper = new QueryWrapper<QhscGangjiaosuoEntity>();
                wrapper.eq("e_name", qhscGangjiaosuo.getEName());
                wrapper.last(" AND  TO_DAYS( NOW( ) ) - TO_DAYS( date) <= 1 ");
                QhscGangjiaosuoEntity gangjiaosuo = gangjiaosuoService.getOne(wrapper);

                tradingUnit = CdFuturesConfigureServiceImpl.setCompany(gangjiaosuo.getName());

                //浮动盈亏 =（当天结算价—昨日结算价）* 委托数量 * 合约面值
                float_pl = algorithm.getFloatingPL(gangjiaosuo.getLastPrice(), gangjiaosuo.getDayUp(), new BigDecimal(tradingUnit), configureEntity.getEntrustNumber());
                accountInfoResp.setFloatingPL(float_pl);
                //资产市值 市值=外汇期货合约价格 * 委托数量 * 合约面值

                //查询持仓信息状态是开仓还是平仓(0.开仓 1.平仓)
                if (configureEntity.getStatus() == 0) {

                    //资产市值 = (开仓或者平仓) * 委托数量 * 合约面值(1份合约=一手，一手=？吨)
                    marketAssets = algorithm.getMarketValue(configureEntity.getEntrustPrice(), configureEntity.getEntrustNumber(), tradingUnit);

                } else if (configureEntity.getStatus() == 1) {
                    marketAssets = algorithm.getMarketValue(configureEntity.getCloseRate(), configureEntity.getEntrustNumber(), tradingUnit);
                }

                accountInfoResp.setMarketAssets(marketAssets);
            } else {
                accountInfoResp.setMarketAssets(marketAssets);
                accountInfoResp.setFloatingPL(float_pl);
            }


            //总资金 = 资产市值 + 占用保证金 + 冻结资金 + 可用资金
            BigDecimal totalAssets = marketAssets.add(accountInfoResp.getDepositBalance().add(accountInfoResp.getBalance().add(accountInfoResp.getForzenBalance())));
            accountInfoResp.setTotalAssets(totalAssets);


            QhscGangjiaosuoEntity qh = gangjiaosuoService.getById(req.getId());

            //序号
            accountInfoResp.setId(req.getId());
            //代码
            accountInfoResp.setEname(qh.getEName());
            //合约名称
            accountInfoResp.setName(qh.getName());
            //交易方向
            accountInfoResp.setTradingDirection(req.getTradingDirection());
            //需要资金

            int unit = CdFuturesConfigureServiceImpl.setCompany(qh.getName());

            //所需资金 = 委托价格 * 委托数量(手数) * 每手交易单位
            BigDecimal totalAmount = algorithm.getFundsRequired(req.getEntrustPrice(), BigDecimal.valueOf(req.getEntrustNumber()), unit);
            accountInfoResp.setFundingNeeded(totalAmount);

            //创建时间
            accountInfoResp.setCreateTime(DateUtils.getNowTime());
            //最后交易日
            //获取最后交易日
            String tradeTime = algorithm.getLastTradingDay(qh.getName());
            accountInfoResp.setTradeTime(tradeTime);*/

        return r;

    }


    /**
     * 期货期权账户
     *
     * @return
     */
    @PostMapping(value = "/getAccount")
    @ApiOperation(value = "期货期权账户", response = CdAccountInfoResp.class)
    public R getAccountInfo() {

        int userId = TokenUtil.getUserId();
        String getDate = template.opsForValue().get("date:userId_" + userId);

        //查询当前账户金额
        CdUserOptionAccountEntity entity = optionAccountService.optionAccountByUerId(userId);

        CdAccountInfoResp accountInfoResp = new CdAccountInfoResp();
        //可用资金
        accountInfoResp.setBalance(entity.getBalance());
        //冻结资金
        accountInfoResp.setForzenBalance(entity.getForzenBalance());
        //占用保证金
        accountInfoResp.setDepositBalance(entity.getDepositBalance());


        //查询当前用户的开仓信息
        QueryWrapper<CdFuturesConfigureEntity> queryWrapper = new QueryWrapper<CdFuturesConfigureEntity>();
        queryWrapper.eq("user_id", userId);

        BigDecimal float_pl = new BigDecimal(0);
        BigDecimal marketAssets = new BigDecimal(0);
        //查询当前用户持仓信息
        CdFuturesConfigureEntity configureEntity = configureService.getOne(queryWrapper);
        if (configureEntity != null) {

            //根据pid查询当前代码
            QhscGangjiaosuoEntity qhscGangjiaosuo = gangjiaosuoService.getById(configureEntity.getPid());

            //根据代码查询当天汇率价格
            QueryWrapper<QhscGangjiaosuoEntity> wrapper = new QueryWrapper<QhscGangjiaosuoEntity>();
            wrapper.eq("e_name", qhscGangjiaosuo.getEName());
            wrapper.eq("date", getDate);
            QhscGangjiaosuoEntity gangjiaosuo = gangjiaosuoService.getOne(wrapper);

            //每手交易单位(合约面值)
            Integer tradingUnit = CdFuturesConfigureServiceImpl.setCompany(gangjiaosuo.getName());

            //浮动盈亏 =（当天结算价—昨日结算价）* 委托数量 * 合约面值
            float_pl = algorithm.getFloatingPL(gangjiaosuo.getLastPrice(), gangjiaosuo.getDayUp(), new BigDecimal(tradingUnit), configureEntity.getEntrustNumber());
            accountInfoResp.setFloatingPL(float_pl);
            //资产市值 市值=外汇期货合约价格 * 委托数量 * 合约面值

            //查询持仓信息状态是开仓还是平仓(0.开仓 1.平仓)
            if (configureEntity.getStatus() == 0) {

                //资产市值 = (开仓或者平仓) * 委托数量 * 合约面值(1份合约=一手，一手=？吨)
                marketAssets = algorithm.getMarketValue(configureEntity.getEntrustPrice(), configureEntity.getEntrustNumber(), tradingUnit);

            } else if (configureEntity.getStatus() == 1) {
                marketAssets = algorithm.getMarketValue(configureEntity.getCloseRate(), configureEntity.getEntrustNumber(), tradingUnit);
            }

            accountInfoResp.setMarketAssets(marketAssets);
        } else {
            accountInfoResp.setMarketAssets(marketAssets);
            accountInfoResp.setFloatingPL(float_pl);
        }


        //总资金 = 资产市值 + 占用保证金 + 冻结资金 + 可用资金
        BigDecimal totalAssets = marketAssets.add(accountInfoResp.getDepositBalance().add(accountInfoResp.getBalance().add(accountInfoResp.getForzenBalance())));
        accountInfoResp.setTotalAssets(totalAssets);


        return R.ok().put("data", accountInfoResp);
    }
}

