package com.msdw.tms.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.msdw.tms.common.utils.R;
import com.msdw.tms.common.utils.TokenUtil;
import com.msdw.tms.entity.CdExperimentalEntity;
import com.msdw.tms.model.req.CdExperimentReq;
import com.msdw.tms.service.CdExperimentalService;
import io.swagger.annotations.*;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;


/**
 * @描述：实验数据控制类
 * @作者: Rong
 * @日期: 2020-10-20
 */
@RestController
@RequestMapping("/chuanda/experimental")
@Api(value = "API - 川大:实验数据", tags = "川大子系统:实验数据")
public class CdExperimentalController {

    @Autowired
    public CdExperimentalService service;

    @Autowired
    public RedisTemplate<String, String> template;

    /**
     * @描述: 实验数据展示
     * @入参:
     * @出参:
     * @作者: Rong
     * @日期: 2020-10-20
     **/
    @PostMapping("/listByCondition")
    @ApiOperation(value = " 实验数据展示", response = CdExperimentalEntity.class)
    public R listByEntity(@RequestParam("type") @ApiParam(value = "实验类型(0正态Var，1历史模拟，2蒙特卡洛)") Integer type) {

        int userId = TokenUtil.getUserId();
        QueryWrapper<CdExperimentalEntity> entity = new QueryWrapper<>();
        entity.eq("type", type);
        entity.eq("user_id", userId);
        List<CdExperimentalEntity> list = service.list(entity);
        return R.ok().put("data", list);
    }


    /**
     * @描述: 保存实验
     * @入参:
     * @出参:
     * @作者: Rong
     * @日期: 2020-09-28
     **/
    @ApiOperation(value = "保存实验")
    @PostMapping(value = "/save", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    @CrossOrigin
    public R save(@RequestBody CdExperimentReq req) {
        int userId = TokenUtil.getUserId();

        CdExperimentalEntity entity = new CdExperimentalEntity();
        String getDate = template.opsForValue().get("date:userId_" + userId);
        //当前时间
        entity.setCreatTime(getDate);
        //汇率类型
        entity.setType(Integer.valueOf(req.getType()));
        //汇率名称
        entity.setExchangeRate(req.getName());
        //样本数据
        entity.setSampleData(req.getSampleData());
        //置信水平
        entity.setConfidenceLevel(req.getConfidenceLevel());
        //持有天数
        entity.setValidPeriod(req.getValidPeriod());

        entity.setUserId(Integer.valueOf(userId));

        //实验类型(0正态Var，1历史模拟，2蒙特卡洛)
        switch (req.getType()) {
            case "0":
            case "1":
                //Var值
                entity.setVarCode(req.getVarCode());
                break;

            case "2":
                //实验次数
                entity.setExperimentsNumber(Integer.valueOf(req.getExperimentsNumber()));
                //汇率价格走势图
                entity.setTrendChart(req.getTrendChart());
                //汇率损益分布图
                entity.setProfitLossDistribution(req.getProfitLossDistribution());
                break;
        }
        boolean result = service.save(entity);
        if (result) {
            return R.ok();
        }
        return R.error();
    }


    /**
     * @描述: 根据主键删除
     * @入参: id
     * @出参:
     * @作者: Rong
     * @日期: 2020-10-20
     **/
    @PostMapping("/delete")
    @ApiOperation(value = "根据主键删除")
    public R delete(@RequestParam("id") @ApiParam(value = "要删除的id") Integer id) {
        service.removeById(id);
        return R.ok();
    }


    /**
     * @描述: 批量删除
     * @入参: id
     * @出参:
     * @作者: Rong
     * @日期: 2020-10-20
     **/
    @PostMapping("/deleteBatchIds")
    @ApiOperation(value = "批量删除")
    public R deleteBatchIds(@RequestParam("ids") @ApiParam(value = "要删除的id列表") List<Integer> ids) {
        boolean result = service.removeByIds(ids);
        if (result) {
            return R.ok();
        }
        return R.error();
    }


    /**
     * @描述: 根据时间、币种条件查询后导出报表
     * @入参:
     * @出参:
     * @作者: Rong
     * @日期: 2020-09-28
     **/
    @ApiOperation(value = "单条导出实验结果")
    @GetMapping(value = "/export", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    @ResponseBody
    @CrossOrigin
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "序号", required = true, dataType = "String", paramType = "query")
    })
    public R export(HttpServletResponse response, @Param(value = "id") Integer id) {

        try {
            //TODO:历史数据下载：只能选取所选择的行情日期之前的数据下载
            R r = service.exportProjectRecord(response, id);

            return r;
        } catch (Exception e) {
            e.printStackTrace();
            return R.error();
        }

    }

    /**
     * 根据类型导出相应实验结果
     *
     * @param response
     * @param ids
     * @return
     */
    @ApiOperation(value = "根据类型导出相应实验结果", response = CdExperimentalEntity.class)
    @GetMapping(value = "/exportByAll", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    @ResponseBody
    @CrossOrigin
    public R exportByAll(HttpServletResponse response, @RequestParam("ids") @ApiParam(value = "要导出的id") List<Integer> ids) {
        try {
            int userId = TokenUtil.getUserId();
            R r = service.exportexportProjectRecordById(response, userId, ids);
            return r;
        } catch (Exception e) {
            e.printStackTrace();
            return R.error();
        }

    }


}

