package com.msdw.tms.controller;

import com.msdw.tms.common.utils.R;
import com.msdw.tms.common.utils.TokenUtil;
import com.msdw.tms.entity.CdFuturesConfigureEntity;
import com.msdw.tms.entity.vo.CdFuturesConfigureEntityVo;
import com.msdw.tms.model.req.CdBuyInReq;
import com.msdw.tms.model.req.CdOpenPositionReq;
import com.msdw.tms.model.resp.CdAfterHedgingResp;
import com.msdw.tms.model.resp.CdBeforeHedgingResp;
import com.msdw.tms.model.resp.CdHoldPositionInfoResp;
import com.msdw.tms.model.resp.CdTransferWarehouseDetailResp;
import com.msdw.tms.service.CdFuturesConfigureService;
import com.msdw.tms.service.QhscGangjiaosuoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;


/**
 * @描述：期货资产头寸配置控制类
 * @作者: Rong
 * @日期: 2020-10-10
 */
@RestController
@RequestMapping("/chuanda/futuresConfigure")
@Api(tags = "川大子系统:开仓平仓相关操作")
public class CdFuturesConfigureController {

    @Autowired
    public CdFuturesConfigureService service;

    @Autowired
    public QhscGangjiaosuoService qhscGangjiaosuoService;

    /**
     * @描述: 开仓
     * @入参:
     * @出参:
     * @作者: Rong
     * @日期: 2020-10-10
     **/
    @PostMapping("/save")
    @ApiOperation(value = "开仓操作")
    public R save(@RequestBody CdOpenPositionReq req) {
        int userId = TokenUtil.getUserId();

        if (req.getEntrustNumber() <= 0) {
            return R.error("委托数量不能小于等于0!");
        }
        CdFuturesConfigureEntity entity = new CdFuturesConfigureEntity();
        entity.setUserId(userId);
        entity.setPid(req.getPid());
        entity.setTradingDirection(req.getTradingDirection());
        entity.setNewestPrice(req.getNewestPrice());
        entity.setEntrustPrice(req.getEntrustPrice());
        entity.setEntrustNumber(req.getEntrustNumber());
        entity.setFundingNeeded(req.getFundingNeeded());


        return service.openPosition(entity);
    }

    /**
     * @描述: 外汇期货持仓情况
     * @入参:
     * @出参:
     * @作者: Rong
     * @日期: 2020-10-12
     **/
    @PostMapping("/findById")
    @ApiOperation(value = "外汇期货持仓情况", response = CdHoldPositionInfoResp.class)
    public R findById() {
        try {
            int userId = TokenUtil.getUserId();
            CdHoldPositionInfoResp entity = service.myPosition(userId);

            if (entity == null) {
                return R.error("暂无持仓信息");
            }
            return R.ok().put("data", entity);

        } catch (Exception e) {
            e.printStackTrace();
            return R.error();
        }
    }

    /**
     * @描述: 平仓
     * @入参:
     * @出参:
     * @作者: Rong
     * @日期: 2020-10-10
     **/
    @PostMapping("/buyIn")
    @ApiOperation(value = "平仓操作", response = CdFuturesConfigureEntity.class)
    public R closePosition(@RequestBody CdBuyInReq req) {
        int userId = TokenUtil.getUserId();
        return service.buyIn(req.getId(), req.getDate(), userId);


    }

    /**
     * @描述: 调仓详情
     * @作者: Rong
     * @日期: 2020-10-14
     **/
    @PostMapping("/detailByTransferWarehouse")
    @ApiOperation(value = "调仓详情", response = CdTransferWarehouseDetailResp.class)
    public R transferWarehouse(@RequestParam("id") @ApiParam(value = "序号") Integer id) {
        try {
            CdTransferWarehouseDetailResp configure = service.transferWarehouseByDetail(id);


            return R.ok().put("data", configure);
        } catch (Exception e) {
            e.printStackTrace();
            return R.error();
        }


    }

    /**
     * @描述: 外汇期货平仓绩效
     * @入参:
     * @出参:
     * @作者: Rong
     * @日期: 2020-10-10
     **/
    @PostMapping("/buyInAchievements")
    @ApiOperation(value = "外汇期货平仓绩效", response = CdFuturesConfigureEntityVo.class)
    public R buyInAchievements(@RequestParam("id") @ApiParam(value = "序号") Integer id) {
        try {
            int userId = TokenUtil.getUserId();
            CdFuturesConfigureEntityVo entity = service.buyInAchievements(id, userId);
            return R.ok().put("data", entity);
        } catch (Exception e) {
            e.printStackTrace();
            return R.error();
        }

    }

    /**
     * 对冲前-外汇现货盈亏
     *
     * @return
     */
    @PostMapping("/beforeHedging")
    @ApiOperation(value = "对冲前-外汇现货盈亏", response = CdBeforeHedgingResp.class)
    public R beforeHedging() {
        try {
            int userId = TokenUtil.getUserId();
            CdBeforeHedgingResp cdBeforeHedgingResp = service.beforeHedging(userId);
            return R.ok().put("data", cdBeforeHedgingResp);
        } catch (Exception e) {
            e.printStackTrace();
            return R.error();
        }

    }

    /**
     * 对冲后损益情况
     * 样本区间：取开仓时间-开仓后的三个月
     *
     * @return
     */
    @PostMapping("/afterHedging")
    @ApiOperation(value = "对冲后损益情况", response = CdAfterHedgingResp.class)
    public R afterHedging() {
        try {
            int userId = TokenUtil.getUserId();
            CdAfterHedgingResp afterHedging = service.afterHedging(userId);
            return R.ok().put("data", afterHedging);
        } catch (Exception e) {
            e.printStackTrace();
            return R.error();
        }

    }

}

