package com.msdw.tms.controller;

import com.msdw.tms.common.utils.R;
import com.msdw.tms.common.utils.TokenUtil;
import com.msdw.tms.common.utils.date.DateUtils;
import com.msdw.tms.entity.CdFuturesConfigureEntity;
import com.msdw.tms.entity.vo.CdFuturesConfigureEntityVo;
import com.msdw.tms.model.resp.CdBeforeHedgingResp;
import com.msdw.tms.model.resp.CdHedgingEfficiencyResp;
import com.msdw.tms.model.resp.CdHedgingResp;
import com.msdw.tms.service.CdFuturesConfigureService;
import com.msdw.tms.service.impl.CdFuturesConfigureServiceImpl;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;


/**
 * @描述：对冲
 * @作者: Rong
 * @日期: 2020-10-21
 */

@RestController
@RequestMapping("/chuanda/hedging")
@Api(value = "API - 川大:对冲", tags = "川大子系统:对冲")
public class CdHedgingController {
    @Autowired
    public CdFuturesConfigureService configureService;

    /**
     * @描述: 对冲
     * @入参:
     * @出参:
     * @作者: Rong
     * @日期: 2020-10-21
     **/
    @PostMapping("/hedgingInfo")
    @ApiOperation(value = "风险对冲核算", response = CdHedgingResp.class)
    public R buyInAchievements(@RequestParam("id") @ApiParam(value = "序号") Integer id) {
        try {

            CdHedgingResp resp = new CdHedgingResp();
            int userId = TokenUtil.getUserId();

            //转换百分比
            DecimalFormat df = new DecimalFormat("0.00%");

            //期货开仓平仓记录
            CdFuturesConfigureEntity entity = configureService.getById(id);

            CdFuturesConfigureEntityVo configure = configureService.buyInAchievements(id, userId);
            configure.setEntrustPrice(entity.getEntrustPrice());

            CdBeforeHedgingResp beforeHedging = configureService.beforeHedging(userId);
            String getCreateTime = DateUtils.dateForByStr(beforeHedging.getCreateTime());
            String getUpdateTime = DateUtils.dateForByStr(beforeHedging.getUpdateTime());

            beforeHedging.setCreateTime(getCreateTime);
            beforeHedging.setUpdateTime(getUpdateTime);


            CdHedgingEfficiencyResp efficiencyResp = new CdHedgingEfficiencyResp();


            //获取期货合约面值
            Integer setCompany = CdFuturesConfigureServiceImpl.setCompany(configure.getName());


            //合约名称
            efficiencyResp.setName(configure.getName().substring(0, 6));


            //现货盈亏
            efficiencyResp.setPromptGoodsPL(beforeHedging.getProfitLoss());

            //期货盈亏
            efficiencyResp.setFuturesPl(configure.getWh_pl());


            //总盈亏 = 外汇期货盈亏 + 外汇现货盈亏
            BigDecimal totalPl = configure.getWh_pl().add(beforeHedging.getProfitLoss());
            efficiencyResp.setTotalPl(totalPl);
            //totalPl = totalPl.divide(new BigDecimal(3), 2, BigDecimal.ROUND_HALF_UP);

            //外汇期货盈亏
            Integer getWh_pl = configure.getWh_pl().intValue();

            //外汇现货盈亏的绝对值
            Integer getProfitLoss = beforeHedging.getProfitLoss().intValue();


            //风险对冲效率（%） = 外汇期货盈亏 / 外汇现货盈亏的绝对值
            double getWh_plRiskEfficiency = getAbsolute(getWh_pl);
            double getProfitLossRiskEfficiency = getAbsolute(getProfitLoss);


            BigDecimal RiskEfficiency = new BigDecimal(getProfitLossRiskEfficiency);
            if (RiskEfficiency.intValue() == 0) {
                efficiencyResp.setRiskEfficiency((0) + "%");
            } else {
                BigDecimal riskEfficiency = new BigDecimal(getWh_plRiskEfficiency).divide(RiskEfficiency, 4, RoundingMode.HALF_UP);
                efficiencyResp.setRiskEfficiency((riskEfficiency.multiply(new BigDecimal(100))) + "%");
            }

            //风险对冲比率（%）=（期货合约面值 * 持仓总量) / 外汇金额

            double num = configure.getEntrustNumber();

            //期货合约面值 * 持仓总量
            double price = setCompany * num;

            /**
             * 如果 风险对冲比率（%）=（期货合约面值 * 持仓总量) /所需要的的金额=(期货合约价*持仓总量*每手)
             * 比如合约每手价格为6.85元，合约面值为 100000 持仓量为1手
             * （100000 * 1)/(6.85*100000 *1)
             */


            //TODO 风险对冲比率（%）=（期货合约面值 * 持仓总量) / 外汇金额(10000000元)
            //外汇金额=所需资金
            //BigDecimal needPrice = BigDecimal.valueOf(setCompany).multiply(BigDecimal.valueOf(configure.getEntrustNumber())).multiply(configure.getEntrustPrice());
            // BigDecimal riskRatio = price.subtract(needPrice);

            double riskRatio = price / 10000000;


            //double percentage2 = riskRatio / setNum;

            //riskRatio转换为%
            efficiencyResp.setRiskRatio(riskRatio + "%");

            BigDecimal need = beforeHedging.getNeededPriceByOpen().add(beforeHedging.getNeededPriceByClose());

            //need = need.divide(new BigDecimal(3), 2, BigDecimal.ROUND_HALF_UP);

            //总盈亏率 = (外汇期货盈亏+外汇现货的盈亏)/(外汇期货所需资金+外汇现货所需资金)
            BigDecimal totalPlRatio = totalPl.divide(need, 4, RoundingMode.HALF_UP);
            efficiencyResp.setTotalPlRatio(totalPlRatio.multiply(new BigDecimal(100)) + "%");


            //外汇期货平仓绩效
            resp.setConfigure(configure);
            //外汇现货盈亏
            resp.setBeforeHedging(beforeHedging);
            //对冲效率
            resp.setHedgingEfficiency(efficiencyResp);

            return R.ok().put("data", resp);
        } catch (Exception e) {
            e.printStackTrace();
            return R.error();
        }

    }

    //获取绝对值
    public static int getAbsolute(Integer num) {
        return num * (1 - ((num >>> 31) << 1));
    }

    public static void main(String[] args) {
        // System.out.println(getAbsolute(5));

        //转换百分比
        DecimalFormat df = new DecimalFormat("0.00%");
        //System.out.println(df.format(0.00196778));

        double a = 1000000;
        double b = 10000000;

        double c = a / b;
        System.out.println(c);


    }

  /*  public static void main(String[] args) {
        String str = "日元兑人民币期货2010";
        System.out.println(str.substring(0,2));
    }*/

}

