package com.msdw.tms.controller;

import com.msdw.tms.common.utils.R;
import com.msdw.tms.common.utils.date.DateUtils;
import com.msdw.tms.entity.CdUserAssetsEntity;
import com.msdw.tms.entity.CdUserOptionAccountEntity;
import com.msdw.tms.service.CdUserAssetsService;
import com.msdw.tms.service.CdUserOptionAccountService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.time.LocalDateTime;


/**
 * @描述：用户银行账户控制类
 * @作者: Rong
 * @日期: 2020-10-10
 */
@RestController
@RequestMapping("/chuanda/userAssets")
@Api(value = "API - 川大:账户信息新增", tags = "川大子系统:账户信息新增")
public class CdUserAssetsController {

    @Autowired
    public CdUserAssetsService service;

    @Autowired
    public CdUserOptionAccountService optionAccountService;

    @Resource
    public RedisTemplate<String, String> template;

    /**
     * @描述: 新增银行账户信息和期权账户信息
     * @作者: Rong
     * @日期: 2020-10-10
     **/
    @PostMapping("/save")
    @ApiOperation(value = "新增银行账户信息和期权账户信息")
    public R save(Integer userId) {

        try {

            String getDate = template.opsForValue().get("date:userId_" + userId);
            //新增银行信息
            CdUserAssetsEntity userAssets = new CdUserAssetsEntity(Long.valueOf(userId), new BigDecimal(100000), new BigDecimal(100000), getDate);

            //新增期权账户信息
            CdUserOptionAccountEntity optionAccount = new CdUserOptionAccountEntity(Long.valueOf(userId), new BigDecimal(100000), new BigDecimal(0), new BigDecimal(0), getDate);

            //TODO 判断期权表和银行账户有没有过userId,没有则新增

            //1.银行账户信息
            CdUserAssetsEntity getBankInfo = service.userAssetsByUerId(userId);
            CdUserOptionAccountEntity getOptionInfo = optionAccountService.optionAccountByUerId(userId);
            if (getBankInfo == null && getOptionInfo == null) {
                //TODO getDate:这里是获取步骤二选中的时间(历史时间)
                userAssets.setCreateTime(getDate);
                userAssets.setUpdateTime(DateUtils.formatTime((LocalDateTime.now())));

                //设置当前时间
                optionAccount.setCreateTime(getDate);
                optionAccount.setUpdateTime(DateUtils.formatTime((LocalDateTime.now())));

                service.addInfo(userAssets);
                optionAccountService.addInfoByOptionAccount(optionAccount);
            }
            return R.ok("新增账户信息成功");
        } catch (Exception e) {
            e.printStackTrace();
            return R.error("账户信息新增失败");
        }


    }


}

