package com.msdw.tms.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.msdw.tms.common.utils.PageUtils;
import com.msdw.tms.common.utils.R;
import com.msdw.tms.common.utils.TokenUtil;
import com.msdw.tms.common.utils.date.DateUtils;
import com.msdw.tms.common.utils.trading_rules.algorithm;
import com.msdw.tms.entity.QhscGangjiaosuoEntity;
import com.msdw.tms.entity.WhscRenminbipinzhongEntity;
import com.msdw.tms.model.req.QhscGangjiaosuoReq;
import com.msdw.tms.model.resp.CdGetDetailByIdResp;
import com.msdw.tms.model.resp.CdGetExchangeRateResp;
import com.msdw.tms.service.QhscGangjiaosuoService;
import com.msdw.tms.service.WhscRenminbipinzhongService;
import com.msdw.tms.service.impl.CdFuturesConfigureServiceImpl;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.*;


/**
 * @描述：控制类
 * @作者: Rong
 * @日期: 2020-10-10
 */
@RestController
@RequestMapping("/chuanda/qhscGangjiaosuo")
@Api(value = "API - 川大:期货合约", tags = "川大子系统:期货合约")
public class QhscGangjiaosuoController {

    @Autowired
    public QhscGangjiaosuoService service;

    @Autowired
    public WhscRenminbipinzhongService rmbService;

    /**
     * @描述: 根据实体属性分页查询
     * @入参:
     * @出参:
     * @作者: Rong
     * @日期: 2020-10-10
     **/
    @PostMapping("/listByEntity")
    @ApiOperation(value = " 列表:根据外币币种分页查询", response = QhscGangjiaosuoEntity.class)
    public R listByEntity(@RequestBody QhscGangjiaosuoReq req) {

        Integer userId = TokenUtil.getUserId();
        if (null == req.getDate()) {
            return R.error("选择时间不能为空");
        }
        PageUtils list = service.queryPageByQh(req.getPageNum(), req.getPageSize(), req.getName(), req.getDate().substring(0, 10));
        return R.ok().put("page", list);
    }


    /**
     * 列表:查询所有外币币种类型
     *
     * @return
     */
    @GetMapping("/getAllCurrency")
    @ApiOperation(value = " 列表:查询所有外币币种类型", response = QhscGangjiaosuoEntity.class)
    public Map<String, Object> getAllCurrency() {

        Integer userId = TokenUtil.getUserId();
        List<QhscGangjiaosuoEntity> list = service.getName();

        QueryWrapper<WhscRenminbipinzhongEntity> wrapper = new QueryWrapper<WhscRenminbipinzhongEntity>();
        wrapper.orderByAsc("date").last("limit 1");
        WhscRenminbipinzhongEntity date = rmbService.getOne(wrapper);


        Iterator<QhscGangjiaosuoEntity> it = list.iterator();
        while (it.hasNext()) {
            String x = it.next().getCurrency();
            if (x.equals("人民币")) {
                it.remove();
            }
        }
        String time = DateUtils.dateForByStr(date.getDate());
        Map<String, Object> map = new HashMap<>();
        map.put("errmessage", "success");
        map.put("data", list);
        map.put("status", 200);
        map.put("startingTime", time);//获取外汇市场人民币品种最前一天的日期


        return map;
        //return R.ok().put("data", list);

    }


    /**
     * 列表:汇率类型
     * 以中间表查询
     *
     * @return
     */
    @GetMapping("/getExchangeRate")
    @ApiOperation(value = " 汇率类型", response = CdGetExchangeRateResp.class)
    public R getExchangeRate() {

        Integer userId = TokenUtil.getUserId();
        //查询汇率类型
        List<WhscRenminbipinzhongEntity> list = rmbService.getExchangeRate();

        //respList:用来保存去除其他字段的list
        List<CdGetExchangeRateResp> respList = new ArrayList<>();

        //查询币种名称
        List<QhscGangjiaosuoEntity> getRMB_Name = service.getName();
        Iterator<QhscGangjiaosuoEntity> it = getRMB_Name.iterator();
        while (it.hasNext()) {
            String x = it.next().getCurrency();
            if (x.equals("人民币")) {
                it.remove();
            }
        }

        for (WhscRenminbipinzhongEntity entity : list) {
            CdGetExchangeRateResp rateResp = new CdGetExchangeRateResp();
            //name:循环出中间价的类型
            String name = entity.getName();

            //循环出人民币币种
            for (QhscGangjiaosuoEntity g : getRMB_Name) {
                String rmbType = g.getCurrency();
                //判断中间价 是否包含 人民币币种
                if (name.contains(rmbType)) {
                    rateResp.setTypeName(name);
                }
            }

            if (rateResp.getTypeName() != null) {
                respList.add(rateResp);
            }

        }


        return R.ok().put("data", respList);

    }


    /**
     * 外汇期货合约类型选择—根据id查看配置头寸信息
     *
     * @return
     */
    @PostMapping("/getDetailById")
    @ApiOperation(value = " 外汇期货合约类型选择—根据id查看配置头寸信息", response = CdGetDetailByIdResp.class)
    public R getDetailById(@RequestParam("id") @ApiParam(value = "要查询的合约id") Integer id) {


        Integer userId = TokenUtil.getUserId();

        //根据id查询合约的信息
        QhscGangjiaosuoEntity exchange = this.service.getById(id);

        //设置每手交易单位(合约面值)
        Integer tradingUnit = 0;
        tradingUnit = CdFuturesConfigureServiceImpl.setCompany(exchange.getName());

        CdGetDetailByIdResp resp = new CdGetDetailByIdResp();
        //最新价格
        resp.setNewestPrice(exchange.getLastPrice());
        //合约名称
        resp.setName(exchange.getName());
        //可交易数量
        resp.setTradableNumber(exchange.getHava());
        //委托价(暂为合约最新价)
        resp.setEntrustPrice(exchange.getLastPrice());
        //委托数量(暂为1手)
        resp.setEntrustNumber(1);


        //:预冻结保证金 = 外汇期货合约价格 * 合约面值 * 手数 * 保证金比例
        BigDecimal getPreFreezingMargin = algorithm.getPreFreezingMargin(exchange.getLastPrice(), new BigDecimal(tradingUnit), exchange.getName().substring(0, 8));
        resp.setPreFreezingMargin(getPreFreezingMargin);

        //:预冻结手续费 = 委托数量（手）* 费用标准
        BigDecimal getPreFreezingServiceCharge = algorithm.getPreFreezingServiceCharge(new BigDecimal(resp.getEntrustNumber()));
        resp.setPreFreezingFrozen(getPreFreezingServiceCharge);

        //:冻结资金 = 预计冻结保证金+预冻结手续费
        BigDecimal frozen_price = getPreFreezingMargin.add(getPreFreezingServiceCharge);
        resp.setFrozenPrice(frozen_price);
        resp.setPid(id);
        resp.setTradingUnit(tradingUnit);
        BigDecimal bond = algorithm.getBond(exchange.getName());
        resp.setTradingBond(bond);

        return R.ok().put("data", resp);


    }


}

