package com.msdw.tms.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.msdw.tms.common.utils.PageUtils;
import com.msdw.tms.common.utils.R;
import com.msdw.tms.common.utils.TokenUtil;
import com.msdw.tms.entity.WhscRenminbipinzhongEntity;
import com.msdw.tms.entity.vo.CdCreatePhotoVo;
import com.msdw.tms.model.req.CdExportReq;
import com.msdw.tms.model.req.CdGetWeekAndMonthInfoReq;
import com.msdw.tms.model.req.CdPageConditionReq;
import com.msdw.tms.model.req.CdSimulationReq;
import com.msdw.tms.model.resp.CdGetWeekAndMonthInfoResp;
import com.msdw.tms.service.WhscRenminbipinzhongService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;
import redis.clients.jedis.Jedis;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * @描述：外汇市场人民币品种控制类
 * @作者: Rong
 * @日期: 2020-09-28
 */

@RestController
@RequestMapping("/chuanda/foreignExchange")
@Api(value = "API - 川大:外汇市场人民币品种", tags = "川大子系统:外汇市场")
public class WhscRenminbipinzhongController {

    @Resource
    public WhscRenminbipinzhongService service;

    /**
     * @描述: 列表:根据时间、币种条件查询
     * @入参:
     * @出参:
     * @作者: Rong
     * @日期: 2020-09-28
     **/
    @ApiOperation(value = "当日外汇行情:根据时间、币种条件查询", response = WhscRenminbipinzhongEntity.class)
    @PostMapping(value = "/pageCondition")
    public R pageCondition(@RequestBody CdPageConditionReq entity) {

        Integer userId = TokenUtil.getUserId();
        //TODO  日期格式化
        String dateFormat = entity.getDate();
        PageUtils list = service.queryPageByRecode(entity.getPageNum(), entity.getPageSize(), entity.getName(), dateFormat, userId);
        return R.ok().put("data", list);
    }


    /**
     * @描述: 查看外汇详情
     * @入参:
     * @出参:
     * @作者: Rong
     * @日期: 2020-09-28
     **/
    @ApiOperation(value = "查看外汇详情", response = WhscRenminbipinzhongEntity.class)
    @PostMapping(value = "/detail")
    public R detail(@RequestParam("id") @ApiParam(value = "序号") Integer id) {
        Integer userId = TokenUtil.getUserId();
        WhscRenminbipinzhongEntity entity = service.getById(id);
        return R.ok().put("data", entity);
    }


    /**
     * @描述: 查看外汇每周/每月的信息
     * @入参:
     * @出参:
     * @作者: Rong
     * @日期: 2020-09-28
     **/
    @ApiOperation(value = "查看外汇每周/每月的信息", response = CdGetWeekAndMonthInfoResp.class)
    @PostMapping(value = "/getWeekAndMonthInfo")
    public Map<String, Object> getWeekAndMonthInfo(@RequestBody CdGetWeekAndMonthInfoReq req) {

        Map<String, Object> map = new HashMap<String, Object>();

        Integer userId = TokenUtil.getUserId();


        //1.根据id查询到该条外汇信息
        WhscRenminbipinzhongEntity entity = service.getById(req.getId());

        //2.根据外汇名称查询该合约一周的信息
        QueryWrapper<WhscRenminbipinzhongEntity> wrapper = new QueryWrapper<>();
        wrapper.eq("name", entity.getName());
        wrapper.last(" and YEARWEEK(date_format(date,'%Y-%m-%d')) = YEARWEEK('" + req.getDate() + "') ORDER BY date ");
        List<WhscRenminbipinzhongEntity> week = service.list(wrapper);

        /*List<CdGetWeekAndMonthInfoResp> getWeek = new ArrayList<CdGetWeekAndMonthInfoResp>();

        for (WhscRenminbipinzhongEntity list : week) {
            CdGetWeekAndMonthInfoResp resp = new CdGetWeekAndMonthInfoResp();
            resp.setDate(list.getDate());
            resp.setLastPrice(list.getLastPrice());
            getWeek.add(resp);

        }*/


        //3.根据外汇名称查询该合约一个月的信息

        QueryWrapper<WhscRenminbipinzhongEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("name", entity.getName());
        queryWrapper.last(" AND DATE_FORMAT( date, '%Y%m' ) = DATE_FORMAT( '" + req.getDate() + "' , '%Y%m' ) ORDER BY date ");
        List<WhscRenminbipinzhongEntity> month = service.list(queryWrapper);

       /* List<CdGetWeekAndMonthInfoResp> getMonth = new ArrayList<>();

        for (WhscRenminbipinzhongEntity list1 : month) {
            CdGetWeekAndMonthInfoResp resp1 = new CdGetWeekAndMonthInfoResp();
            resp1.setDate(list1.getDate());
            resp1.setLastPrice(list1.getLastPrice());
            getMonth.add(resp1);

        }*/

        map.put("errmessage", "success");
        map.put("status", 200);
        map.put("week", week);
        map.put("month", month);
        return map;

    }


    /**
     * 根据时间、币种条件查询后导出报表
     *
     * @param exportReq
     * @param response
     * @return
     */
    @ApiOperation(value = "根据时间、币种条件查询后导出报表", notes = "根据时间、币种条件查询后导出报表")
    @RequestMapping(value = "/export", method = RequestMethod.GET, produces = "application/json")
    public R export(CdExportReq exportReq, HttpServletResponse response) {
        try {
            //TODO:历史数据下载：只能选取所选择的行情日期之前的数据下载
            R r = service.exportProjectRecord(response, exportReq.getName(), exportReq.getStartTime(), exportReq.getEndTime());
            return r;
        } catch (Exception e) {
            e.printStackTrace();
            return R.error();
        }

    }


    /**
     * @描述: 根据选择的模拟法传值获取相应数据
     * @入参:
     * @出参:
     * @作者: Rong
     * @日期: 2020-09-28
     **/
    @ApiOperation(value = "根据选择的模拟法传值获取相应数据", response = CdCreatePhotoVo.class)
    @PostMapping(value = "/simulation", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    @CrossOrigin
    public R simulation(@RequestBody CdSimulationReq req) {

        try {
            Integer userId = TokenUtil.getUserId();
            //TODO:历史数据下载：只能选取所选择的行情日期之前的数据下载
            R r = service.getInfobyId(req.getName(), req.getStartTime(), req.getEndTime(), req.getConfidenceLevel(), req.getValidity(), req.getType(), req.getExperimentsNumber());

            return r;
        } catch (Exception e) {
            e.printStackTrace();
            return R.error();
        }


    }

    public static void main(String[] args) {
        //连接本地的 Redis 服务
        Jedis jedis = new Jedis("localhost");
        System.out.println("连接成功");
        //查看服务是否运行
        System.out.println("服务正在运行: " + jedis.ping());
    }


}
