package com.msdw.tms.entity;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.math.BigDecimal;

/**
 * @描述：建仓记录表
 * @作者: Rong
 * @日期: 2020-10-12
 */
@Data
@ApiModel(value = "建仓记录表")
@TableName("cd_futures_configure")
public class CdFuturesConfigureEntity implements Serializable {

    private static final long serialVersionUID = 1L;

    @ApiModelProperty(value = "主键", required = false)
    @TableId(type = IdType.AUTO)
    private Integer id;

    @ApiModelProperty(value = "用户id")
    @NotNull(message = "用户id不能为空")
    private Integer userId;

    @ApiModelProperty(value = "合约表主键")
    @NotNull(message = "合约表主键不能为空")
    private Integer pid;

    @ApiModelProperty(value = "交易方向(0空头,1多头)")
    @NotBlank(message = "交易方向不能为空")
    private Integer tradingDirection;

    @ApiModelProperty(value = "最新价格")
    @NotBlank(message = "最新价格不能为空")
    private BigDecimal newestPrice;

    @ApiModelProperty(value = "委托价格")
    @NotNull(message = "委托价格不能为空")
    private BigDecimal entrustPrice;

    @ApiModelProperty(value = "委托数量")
    @NotNull(message = "委托数量不能为空")
    private Integer entrustNumber;

    @ApiModelProperty(value = "可交易数量")
    @NotNull(message = "可交易数量不能为空")
    private BigDecimal tradableNumber;

    @ApiModelProperty(value = "预冻结保证金")
    private BigDecimal preFreezingMargin;

    @ApiModelProperty(value = "预冻结手续费")
    private BigDecimal preFreezingFrozen;

    @ApiModelProperty(value = "所需资金")
    private BigDecimal fundingNeeded;

    @ApiModelProperty(value = "冻结资金")
    private BigDecimal frozenPrice;

    @ApiModelProperty(value = "0.开仓 1.平仓")
    private Integer status;

    @ApiModelProperty(value = "是否自动平仓（0否 1是）")
    private Integer type;

    @ApiModelProperty(value = "浮动盈亏")
    private BigDecimal floatingPl;

    @ApiModelProperty(value = "开仓时间")
    @NotBlank(message = "开仓时间不能为空")
    private String createTime;

    @ApiModelProperty(value = "平仓时间")
    @NotBlank(message = "平仓时间不能为空")
    private String updateTime;

    @ApiModelProperty(value = "最后交易日")
    @NotBlank(message = "最后交易日不能为空")
    private String tradeTime;


    @ApiModelProperty(value = "平仓价格")
    @NotBlank(message = "平仓价格不能为空")
    private BigDecimal closeRate;

    //transient设置非数据库字段
    /**
     * 浮动盈亏
     */
    //private transient String floatingPL;

    /**
     * 名称
     */
    @ApiModelProperty(value = "名称")
    private transient String name;

    /**
     * 代码
     */
    @ApiModelProperty(value = "代码")
    private transient String ename;


    /**
     * 当前最新价
     */
    @ApiModelProperty(value = "当前最新价")
    private transient BigDecimal lastPrice;

    /**
     * 涨跌幅
     */
    @ApiModelProperty(value = "涨跌幅")
    private transient String u;


    /**
     * 昨收
     */
    @ApiModelProperty(value = "昨结")
    private transient BigDecimal dayUp;

    //买卖方向为开仓时候:可交易数量为Python表的持仓总量
    @ApiModelProperty(value = "买卖方向为开仓时候:可交易数量为Python表的持仓总量")
    private transient BigDecimal tradableQuantityByBuy;

    //买卖方向为平仓时候:可交易数量为自己的持仓总量
    @ApiModelProperty(value = "买卖方向为平仓时候:可交易数量为自己的持仓总量")
    private transient BigDecimal tradableQuantityBySell;

    //当前账户期权可用资金
    @ApiModelProperty(value = "当前期权账户可用资金")
    private transient BigDecimal availableFunds;


}