package com.msdw.tms.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.msdw.tms.common.utils.R;
import com.msdw.tms.common.utils.trading_rules.algorithm;
import com.msdw.tms.dao.*;
import com.msdw.tms.entity.*;
import com.msdw.tms.model.req.CdGetAccountInfoReq;
import com.msdw.tms.model.resp.CdAccountInfoResp;
import com.msdw.tms.model.resp.CdTransferRecordsResp;
import com.msdw.tms.service.CdBankStatementsService;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

/**
 * @描述：账户流水记录 服务类
 * @作者: Rong
 * @日期: 2020-10-15
 */
@Service("cdBankStatementsService")
public class CdBankStatementsServiceImpl extends ServiceImpl<CdBankStatementsDao, CdBankStatementsEntity> implements CdBankStatementsService {


    @Resource
    private CdBankStatementsDao dao;

    //期权账户
    @Resource
    private CdUserOptionAccountDao accountDao;


    //银行账户
    @Resource
    private CdUserAssetsDao bankDao;

    @Resource
    private QhscGangjiaosuoDao gangjiaosuoDao;

    @Resource
    private CdFuturesConfigureDao configureDao;
    @Resource
    public RedisTemplate<String, String> template;


    /**
     * 转账后添加转账流水
     * 对相应的账户做修改操作
     *
     * @param
     * @param
     * @return
     */
    @Override
    //@Transactional(propagation = Propagation.NESTED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
    public boolean modify(Integer userId, Integer transferOut, BigDecimal amountMoney) {
        try {

            String getDate = template.opsForValue().get("date:userId_" + userId);

            //获取当前银行可用资金
            QueryWrapper<CdUserAssetsEntity> queryWrapper = new QueryWrapper<CdUserAssetsEntity>();
            queryWrapper.eq("user_id", userId);
            //getBankBalance:获取到当前银行可用资金
            BigDecimal getBankBalance = bankDao.selectOne(queryWrapper).getBalance();

            //获取当前期权账户可用资金
            QueryWrapper<CdUserOptionAccountEntity> queryWrapper2 = new QueryWrapper<CdUserOptionAccountEntity>();
            queryWrapper2.eq("user_id", userId);
            //getBalance:获取到当前期权账户可用资金
            BigDecimal getBalance = accountDao.selectOne(queryWrapper2).getBalance();


            UpdateWrapper<CdUserAssetsEntity> updateWrapper1 = new UpdateWrapper<CdUserAssetsEntity>();
            UpdateWrapper<CdUserOptionAccountEntity> updateWrapper2 = new UpdateWrapper<CdUserOptionAccountEntity>();
            CdBankStatementsEntity bankStatements1 = new CdBankStatementsEntity();
            CdBankStatementsEntity bankStatements2 = new CdBankStatementsEntity();

            updateWrapper1.eq("user_id", userId);
            updateWrapper2.eq("user_id", userId);

            if (transferOut == 1) {
                /* 操作1:银行账户扣钱 期权账户加钱 同时添加流水记录*/


                //银行账户扣钱操作
                updateWrapper1.set("balance", getBankBalance.subtract(amountMoney));
                int updateBank = bankDao.update(null, updateWrapper1);


                //期权账户加钱操作
                updateWrapper2.set("balance", getBalance.add(amountMoney));
                int updateAssets = accountDao.update(null, updateWrapper2);


                //添加银行账户流水记录
                bankStatements1.setType(1);//类型（1银行账户 2期货账户）
                bankStatements1.setCoinType("人民币");
                bankStatements1.setBeforeAmount(getBankBalance);//改变前金额
                bankStatements1.setAmountMoney(amountMoney);//金额
                bankStatements1.setAfterAmount(getBankBalance.subtract(amountMoney));
                bankStatements1.setTransferType(1);//流水类型(1:转账 2:流水)
                bankStatements1.setRemark("转账");
                bankStatements1.setUserId(userId);
                bankStatements1.setCreateTime(getDate);
                int addBankStatements1 = baseMapper.insert(bankStatements1);

                //添加期权账户流水记录
                bankStatements2.setType(2);//类型（1银行账户 2期货账户）
                bankStatements2.setCoinType("人民币");
                bankStatements2.setBeforeAmount(getBalance);//改变前金额
                bankStatements2.setAmountMoney(amountMoney);//金额
                bankStatements2.setAfterAmount(getBalance.add(amountMoney));
                bankStatements2.setTransferType(1);//流水类型(1:转账 2:流水)
                bankStatements2.setUserId(userId);
                bankStatements2.setRemark("入账");
                bankStatements2.setCreateTime(getDate);
                int addBankStatements2 = baseMapper.insert(bankStatements2);


                if (updateAssets <= 0 && updateBank <= 0 && addBankStatements1 <= 0 && addBankStatements2 <= 0) {
                    return false;
                }


            } else if (transferOut == 2) {
                //以期权账户转出到银行账户
                /* 操作2:期权账户扣钱 银行账户加钱 同时添加流水记录*/


                //期权账户扣钱
                updateWrapper2.set("balance", getBalance.subtract(amountMoney));
                updateWrapper2.eq("user_id", userId);
                int updateAssets = accountDao.update(null, updateWrapper2);


                //银行账户加钱
                updateWrapper1.set("balance", getBankBalance.add(amountMoney));
                updateWrapper1.eq("user_id", userId);
                int updateBank = bankDao.update(null, updateWrapper1);


                //添加银行账户流水记录

                bankStatements1.setType(2);//类型（1银行账户 2期货账户）
                bankStatements1.setCoinType("人民币");
                bankStatements1.setBeforeAmount(getBankBalance);//改变前金额
                bankStatements1.setAmountMoney(amountMoney);//金额
                bankStatements1.setAfterAmount(getBankBalance.add(amountMoney));
                bankStatements1.setTransferType(1);//流水类型(1:转账 2:流水)
                bankStatements1.setRemark("转账");
                bankStatements1.setCreateTime(getDate);
                bankStatements1.setUserId(userId);
                int addBankStatements1 = baseMapper.insert(bankStatements1);

                //添加期权账户流水记录

                bankStatements2.setType(1);//类型（1银行账户 2期货账户）
                bankStatements2.setCoinType("人民币");
                bankStatements2.setBeforeAmount(getBalance);//改变前金额
                bankStatements2.setAmountMoney(amountMoney);//金额
                bankStatements2.setAfterAmount(getBalance.subtract(amountMoney));
                bankStatements2.setTransferType(1);//流水类型(1:转账 2:流水)
                bankStatements2.setRemark("入账");
                bankStatements2.setCreateTime(getDate);
                bankStatements2.setUserId(userId);
                int addBankStatements2 = baseMapper.insert(bankStatements2);


                if (updateAssets <= 0 && updateBank <= 0 && addBankStatements1 <= 0 && addBankStatements2 <= 0) {
                    return false;
                }


            }
            return true;
        } catch (Exception e) {
            e.getMessage();
            return false;
        }


    }

    /**
     * 转账交易记录
     *
     * @param
     * @param
     * @param userId
     * @param startTime
     * @param endTime
     * @return
     */
    @Override
    public List<CdTransferRecordsResp> queryPageByRecode(String userId, String startTime, String endTime) {
        //分页对象
        QueryWrapper<CdBankStatementsEntity> queryWrapper = new QueryWrapper<CdBankStatementsEntity>();
        //判断条件值是否为空,如果不为空,拼接条件
        if (userId != null) {
            queryWrapper.eq("user_id", userId);
        }


        if (startTime != null && endTime != null) {
            if (!startTime.isEmpty() && !endTime.isEmpty()) {
                queryWrapper.ge("create_time", startTime)
                        .le("create_time", endTime);
            }
        }

        //流水类型(1:转账 2:流水)
        queryWrapper.eq("transfer_type", 1);
        queryWrapper.eq("remark", "转账");

        //获取list
        List<CdBankStatementsEntity> list = baseMapper.selectList(queryWrapper);

        //添加进新的集合
        List<CdTransferRecordsResp> recordsReqs = new ArrayList<>();


        //类型（1银行账户 2期货账户）
        for (CdBankStatementsEntity entity : list) {
            CdTransferRecordsResp req = new CdTransferRecordsResp();
            if (entity.getType() == 1) {
                req.setOut("银行账户");
                req.setEnter("商品期货保证金账户");
            } else if (entity.getType() == 2) {
                req.setOut("商品期货保证金账户");
                req.setEnter("银行账户");

            }
            req.setCoinType("人民币");
            req.setAmountMoney(entity.getAmountMoney());
            req.setCreateTime(entity.getCreateTime());
            req.setType("自由转账");
            recordsReqs.add(req);
        }
        return recordsReqs;
    }

    /**
     * 查看期权账户时候返回头寸配置信息
     *
     * @param req
     * @return
     */
    @Override
    public R getAccountInfoReturnOpen(CdGetAccountInfoReq req, Integer userId) {

        String getDate = template.opsForValue().get("date:userId_" + userId);

        QueryWrapper<CdUserOptionAccountEntity> account = new QueryWrapper<CdUserOptionAccountEntity>();
        account.eq("user_id", userId);

        //查询当前账户金额
        CdUserOptionAccountEntity entity = accountDao.selectOne(account);

        CdAccountInfoResp accountInfoResp = new CdAccountInfoResp();
        //可用资金
        accountInfoResp.setBalance(entity.getBalance());
        //冻结资金
        accountInfoResp.setForzenBalance(entity.getForzenBalance());
        //占用保证金
        accountInfoResp.setDepositBalance(entity.getDepositBalance());

        //每手交易单位(合约面值)
        Integer tradingUnit = 0;

        //查询当前用户的开仓信息
        QueryWrapper<CdFuturesConfigureEntity> queryWrapper = new QueryWrapper<CdFuturesConfigureEntity>();
        queryWrapper.eq("user_id", userId);

        BigDecimal float_pl = new BigDecimal(0);
        BigDecimal marketAssets = new BigDecimal(0);
        //查询当前用户持仓信息
        CdFuturesConfigureEntity configureEntity = configureDao.selectOne(queryWrapper);
        if (configureEntity != null) {

            //根据pid查询当前代码
            QhscGangjiaosuoEntity qhscGangjiaosuo = gangjiaosuoDao.selectById(configureEntity.getPid());

            //根据代码查询当天汇率价格
            QueryWrapper<QhscGangjiaosuoEntity> wrapper = new QueryWrapper<QhscGangjiaosuoEntity>();
            wrapper.eq("e_name", qhscGangjiaosuo.getEName());
            wrapper.eq("date", getDate);
            QhscGangjiaosuoEntity gangjiaosuo = gangjiaosuoDao.selectOne(wrapper);

            tradingUnit = CdFuturesConfigureServiceImpl.setCompany(gangjiaosuo.getName());

            //浮动盈亏 =（当天结算价—昨日结算价）* 委托数量 * 合约面值
            float_pl = algorithm.getFloatingPL(gangjiaosuo.getLastPrice(), gangjiaosuo.getDayUp(), new BigDecimal(tradingUnit), configureEntity.getEntrustNumber());
            accountInfoResp.setFloatingPL(float_pl);
            //资产市值 市值=外汇期货合约价格 * 委托数量 * 合约面值

            //查询持仓信息状态是开仓还是平仓(0.开仓 1.平仓)
            if (configureEntity.getStatus() == 0) {

                //资产市值 = (开仓或者平仓) * 委托数量 * 合约面值(1份合约=一手，一手=？吨)
                marketAssets = algorithm.getMarketValue(configureEntity.getEntrustPrice(), configureEntity.getEntrustNumber(), tradingUnit);

            } else if (configureEntity.getStatus() == 1) {
                marketAssets = algorithm.getMarketValue(configureEntity.getCloseRate(), configureEntity.getEntrustNumber(), tradingUnit);
            }

            accountInfoResp.setMarketAssets(marketAssets);
        } else {
            accountInfoResp.setMarketAssets(marketAssets);
            accountInfoResp.setFloatingPL(float_pl);
        }


        //总资金 = 资产市值 + 占用保证金 + 冻结资金 + 可用资金
        BigDecimal totalAssets = marketAssets.add(accountInfoResp.getDepositBalance().add(accountInfoResp.getBalance().add(accountInfoResp.getForzenBalance())));
        accountInfoResp.setTotalAssets(totalAssets);


        QhscGangjiaosuoEntity qh = gangjiaosuoDao.selectById(req.getId());

        //序号
        accountInfoResp.setId(req.getId());
        //代码
        accountInfoResp.setEname(qh.getEName());
        //合约名称
        accountInfoResp.setName(qh.getName());
        //交易方向
        accountInfoResp.setTradingDirection(req.getTradingDirection());
        //需要资金

        int unit = CdFuturesConfigureServiceImpl.setCompany(qh.getName());

        //所需资金 = 委托价格 * 委托数量(手数) * 每手交易单位
        BigDecimal totalAmount = algorithm.getFundsRequired(req.getEntrustPrice(), BigDecimal.valueOf(req.getEntrustNumber()), unit);
        accountInfoResp.setFundingNeeded(totalAmount);

        //创建时间
        accountInfoResp.setCreateTime(getDate);
        //最后交易日
        //获取最后交易日
        String tradeTime = algorithm.getLastTradingDay(qh.getName());
        accountInfoResp.setTradeTime(tradeTime);

        return R.ok().put("data", accountInfoResp);

    }


}



