package com.msdw.tms.service.impl;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.msdw.tms.common.utils.R;
import com.msdw.tms.dao.CdExperimentalDao;
import com.msdw.tms.entity.CdExperimentalEntity;
import com.msdw.tms.entity.vo.CdExperimentalEntityVo;
import com.msdw.tms.service.CdExperimentalService;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;

/**
 * @描述：实验数据 服务类
 * @作者: Rong
 * @日期: 2020-10-20
 */
@Service("cdExperimentalService")
public class CdExperimentalServiceImpl extends ServiceImpl<CdExperimentalDao, CdExperimentalEntity> implements CdExperimentalService {

    /**
     * 单条导出实验结果
     *
     * @param response
     * @param id
     * @return
     * @throws Exception
     */
    @Override
    public R exportProjectRecord(HttpServletResponse response, Integer id) throws Exception {

        try {

            CdExperimentalEntity entity = baseMapper.selectById(id);
            if (entity == null) {
                return R.ok("暂无此数据");
            }

            QueryWrapper<CdExperimentalEntity> wrapper = new QueryWrapper<CdExperimentalEntity>();
            wrapper.eq("id", id);
            List<CdExperimentalEntity> list = baseMapper.selectList(wrapper);
            switch (entity.getType()) {

                case 0:
                case 1:

                    if (list != null) {

                        // 告诉浏览器用什么软件可以打开此文件
                        response.setHeader("content-Type", "application/vnd.ms-excel");
                        // 下载文件的默认名称
                        response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode("实验数据", "UTF-8") + ".xls");
                        //编码
                        response.setCharacterEncoding("UTF-8");
                        Workbook workbook = ExcelExportUtil.exportExcel(new ExportParams(), CdExperimentalEntity.class, list);
                        workbook.write(response.getOutputStream());
                    }

                    break;
                case 2:


                    CdExperimentalEntity entity1 = baseMapper.selectById(id);
                    CdExperimentalEntityVo vo = new CdExperimentalEntityVo();

                    vo.setId(entity1.getId());
                    vo.setExchangeRate(entity1.getExchangeRate());
                    vo.setSampleData(entity1.getSampleData());
                    vo.setConfidenceLevel(entity1.getConfidenceLevel());
                    vo.setValidPeriod(entity1.getValidPeriod());
                    vo.setTrendChart(entity1.getTrendChart());
                    vo.setProfitLossDistribution(entity1.getProfitLossDistribution());
                    vo.setCreatTime(entity1.getCreatTime());

                    List<CdExperimentalEntityVo> voList = new ArrayList<>();
                    voList.add(vo);


                    // 告诉浏览器用什么软件可以打开此文件
                    response.setHeader("content-Type", "application/vnd.ms-excel");
                    // 下载文件的默认名称
                    response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode("实验数据(蒙特卡洛)", "UTF-8") + ".xls");
                    //编码
                    response.setCharacterEncoding("UTF-8");
                    Workbook workbook = ExcelExportUtil.exportExcel(new ExportParams(), CdExperimentalEntityVo.class, voList);
                    workbook.write(response.getOutputStream());


                    break;

            }


        } catch (IOException e) {
            e.printStackTrace();
            return R.error("出现异常:" + e.getMessage());

        }
        return R.ok();
    }

    /**
     * 根据选择的类型导出实验结果
     *
     * @param response
     * @param userId
     * @return
     * @throws Exception 要导出的类型-实验类型(0正态Var，1历史模拟，2蒙特卡洛)"
     */
    @Override
    public R exportexportProjectRecordByAll(HttpServletResponse response, Integer userId, Integer type) throws Exception {

        //查询该用户做过的实验
        QueryWrapper<CdExperimentalEntity> queryWrapper = new QueryWrapper<CdExperimentalEntity>();
        queryWrapper.eq("user_id", userId);
        queryWrapper.eq("type", type);
        List<CdExperimentalEntity> list = baseMapper.selectList(queryWrapper);

        if (list == null) {
            return R.ok("没有该数据,无法导出Excel。");
        }
        List<CdExperimentalEntityVo> voList = new ArrayList<>();
        switch (type) {

            case 0:
            case 1:


                // 告诉浏览器用什么软件可以打开此文件
                response.setHeader("content-Type", "application/vnd.ms-excel");
                // 下载文件的默认名称
                response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode("实验数据", "UTF-8") + ".xls");
                //编码
                response.setCharacterEncoding("UTF-8");
                Workbook workbook = ExcelExportUtil.exportExcel(new ExportParams(), CdExperimentalEntity.class, list);
                workbook.write(response.getOutputStream());


                break;
            case 2:


                for (CdExperimentalEntity experimentalEntity : list) {
                    CdExperimentalEntityVo vo = new CdExperimentalEntityVo();
                    System.out.println("打印:>>>>>>>" + experimentalEntity.getId());
                    vo.setId(experimentalEntity.getId());
                    vo.setExchangeRate(experimentalEntity.getExchangeRate());
                    vo.setSampleData(experimentalEntity.getSampleData());
                    vo.setConfidenceLevel(experimentalEntity.getConfidenceLevel());
                    vo.setValidPeriod(experimentalEntity.getValidPeriod());
                    vo.setTrendChart(experimentalEntity.getTrendChart());
                    vo.setProfitLossDistribution(experimentalEntity.getProfitLossDistribution());
                    vo.setCreatTime(experimentalEntity.getCreatTime());

                    System.out.println("赋值前打印:" + vo);

                    voList.add(vo);
                }


                // 告诉浏览器用什么软件可以打开此文件
                response.setHeader("content-Type", "application/vnd.ms-excel");
                // 下载文件的默认名称
                response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode("实验数据(蒙特卡洛)", "UTF-8") + ".xls");
                //编码
                response.setCharacterEncoding("UTF-8");
                Workbook workbook1 = ExcelExportUtil.exportExcel(new ExportParams(), CdExperimentalEntityVo.class, voList);
                workbook1.write(response.getOutputStream());
                break;

        }

        return null;

    }

    /**
     * 根据选择的id导出实验
     *
     * @param response
     * @param userId
     * @param ids
     * @return
     * @throws Exception
     */
    @Override
    public R exportexportProjectRecordById(HttpServletResponse response, Integer userId, List<Integer> ids) throws Exception {

        List<CdExperimentalEntity> list = this.listByIds(ids);
        if (list == null) {
            return R.ok("没有该数据,无法导出Excel。");
        }
        List<CdExperimentalEntityVo> voList = new ArrayList<>();
        for (CdExperimentalEntity experimental : list) {
            switch (experimental.getType()) {
                case 0:
                case 1:


                    // 告诉浏览器用什么软件可以打开此文件
                    response.setHeader("content-Type", "application/vnd.ms-excel");
                    // 下载文件的默认名称
                    response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode("实验数据", "UTF-8") + ".xls");
                    //编码
                    response.setCharacterEncoding("UTF-8");
                    Workbook workbook = ExcelExportUtil.exportExcel(new ExportParams(), CdExperimentalEntity.class, list);
                    workbook.write(response.getOutputStream());


                    break;
                case 2:

                    for (CdExperimentalEntity experimentalEntity : list) {
                        CdExperimentalEntityVo vo = new CdExperimentalEntityVo();
                        System.out.println("打印:>>>>>>>" + experimentalEntity.getId());
                        vo.setId(experimentalEntity.getId());
                        vo.setExchangeRate(experimentalEntity.getExchangeRate());
                        vo.setSampleData(experimentalEntity.getSampleData());
                        vo.setConfidenceLevel(experimentalEntity.getConfidenceLevel());
                        vo.setValidPeriod(experimentalEntity.getValidPeriod());
                        vo.setTrendChart(experimentalEntity.getTrendChart());
                        vo.setProfitLossDistribution(experimentalEntity.getProfitLossDistribution());
                        vo.setCreatTime(experimentalEntity.getCreatTime());

                        System.out.println("赋值前打印:" + vo);

                        voList.add(vo);
                    }


                    // 告诉浏览器用什么软件可以打开此文件
                    response.setHeader("content-Type", "application/vnd.ms-excel");
                    // 下载文件的默认名称
                    response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode("实验数据(蒙特卡洛)", "UTF-8") + ".xls");
                    //编码
                    response.setCharacterEncoding("UTF-8");
                    Workbook workbook1 = ExcelExportUtil.exportExcel(new ExportParams(), CdExperimentalEntityVo.class, voList);
                    workbook1.write(response.getOutputStream());
                    break;
            }
        }
        return null;


    }


}



