package com.msdw.tms.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.msdw.tms.common.utils.MoneyConversionUtil;
import com.msdw.tms.common.utils.R;
import com.msdw.tms.common.utils.date.DateUtils;
import com.msdw.tms.common.utils.py.PyUtil;
import com.msdw.tms.common.utils.trading_rules.algorithm;
import com.msdw.tms.dao.*;
import com.msdw.tms.entity.*;
import com.msdw.tms.entity.vo.CdFuturesConfigureEntityVo;
import com.msdw.tms.model.resp.CdAfterHedgingResp;
import com.msdw.tms.model.resp.CdBeforeHedgingResp;
import com.msdw.tms.model.resp.CdHoldPositionInfoResp;
import com.msdw.tms.model.resp.CdTransferWarehouseDetailResp;
import com.msdw.tms.service.CdFuturesConfigureService;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @描述：期货资产头寸配置 服务类
 * @作者: Rong
 * @日期: 2020-10-10
 */
@Service("cdFuturesConfigureService")
public class CdFuturesConfigureServiceImpl extends ServiceImpl<CdFuturesConfigureDao, CdFuturesConfigureEntity> implements CdFuturesConfigureService {


    @Resource
    public QhscGangjiaosuoDao qhscGangjiaosuoDao;

    @Resource
    public CdFuturesConfigureDao futuresConfigureDao;

    @Resource
    public CdUserOptionAccountDao accountDao;

    @Resource
    public CdBankStatementsDao bankStatementsDao;

    @Resource
    public CdTransferWarehouseRecordDao transferWarehouseRecordDao;

    @Resource
    public RedisTemplate<String, String> template;
    @Resource
    public WhscRenminbipinzhongDao renminbipinzhongDao;

    //开仓处理
    public R openPosition(CdFuturesConfigureEntity entity) {
        try {

            //判断返回上一步重新开仓的情况下

            //1.查询该用户有没有建仓信息
            QueryWrapper<CdFuturesConfigureEntity> queryWrapper = new QueryWrapper<CdFuturesConfigureEntity>();
            queryWrapper.eq("user_id", entity.getUserId());
            CdFuturesConfigureEntity check = baseMapper.selectOne(queryWrapper);

            //check不为空的情况下说明有开仓信息 再看有没有修改参数
            // 修改了就删除当前的开仓信息
            if (check != null) {
                //先查看流水表要返回的金额
                QueryWrapper<CdBankStatementsEntity> wrapper = new QueryWrapper<>();
                wrapper.eq("user_id", entity.getUserId());
                wrapper.eq("transfer_type", "2");
                wrapper.eq("type", "2").last("limit 0,1");
                CdBankStatementsEntity get = bankStatementsDao.selectOne(wrapper);

                //获取到第一条改变前的金额
                if (get != null) {
                    //将金额回滚到改变前的金额

                    //做修改账户金额操作
                    UpdateWrapper<CdUserOptionAccountEntity> updateWrapper2 = new UpdateWrapper<CdUserOptionAccountEntity>();
                    updateWrapper2.set("balance", get.getBeforeAmount());
                    //修改占用保证金  getPreFreezingMargin
                    updateWrapper2.set("deposit_balance", 0);
                    updateWrapper2.set("forzen_balance", 0);
                    updateWrapper2.eq("user_id", entity.getUserId());
                    int updateAssets = accountDao.update(null, updateWrapper2);

                    //删除(开仓、平仓、调仓)操作流水表

                    QueryWrapper<CdTransferWarehouseRecordEntity> delRecord = new QueryWrapper<>();
                    delRecord.eq("cid", check.getId());
                    int delRecodeByOpen = transferWarehouseRecordDao.delete(delRecord);

                    //删除头寸表记录
                    int delOpen = baseMapper.deleteById(check.getId());


                    //删除账户流水表记录
                    Map<String, Object> map = new HashMap<>();
                    //删除类型为流水的 : (1:转账 2:流水)
                    map.put("transfer_type", "2");
                    map.put("user_id", entity.getUserId());

                    int delAccount = bankStatementsDao.deleteByMap(map);

                    if (updateAssets <= 0 && delRecodeByOpen <= 0 && delOpen <= 0 && delAccount <= 0) {
                        return R.error();
                    }
                }


            }


            String getDate = template.opsForValue().get("date:userId_" + entity.getUserId());
            //python表:根据合约id查询合约相关数据
            QhscGangjiaosuoEntity qh = qhscGangjiaosuoDao.selectById(entity.getPid());

            if (qh == null) {
                return R.error("暂时没有该期货合约的最新价");
            }

            //委托价格，委托数量为空默认为合约当前最新价格，委托数量为1手
            if (entity.getEntrustPrice() == null && entity.getEntrustNumber() == null) {
                //设置委托价格
                entity.setEntrustPrice(qh.getLastPrice());
                //设置委托数量
                entity.setEntrustNumber(1);

            } else if (entity.getEntrustPrice() != null && entity.getEntrustNumber() == null) {
                //委托数量不为空时，委托数量为null 默认1手

                //设置委托价格
                entity.setEntrustPrice(entity.getEntrustPrice());
                //设置委托数量
                entity.setEntrustNumber(1);
            } else {
                //根据前端传的值设置
                entity.setEntrustPrice(entity.getEntrustPrice());
                entity.setEntrustNumber(entity.getEntrustNumber());

            }
            //设置最新价格
            entity.setNewestPrice(qh.getLastPrice());

            //委托价
            double entrustPrice = entity.getEntrustPrice().doubleValue();

            //最新价
            double newestPrice = entity.getNewestPrice().doubleValue();

            /**
             * 期货的开仓成交规则：
             * 多方情况下，委托价格大于等于最新价，以最新价成交；
             * 空方情况下，委托价格小于等于最新价，以最新价成交。
             */
            //交易方向(0空头,1多头)
            switch (entity.getTradingDirection()) {

                case 0:
                    //空方情况下:委托价格小于等于最新价，以最新价成交
                    // entrustPrice > newestPrice
                    if (entrustPrice > newestPrice) {
                        return R.error("开仓失败,委托价格大于最新价");
                    }
                    break;
                case 1:
                    //多方情况下:委托价格大于等于最新价，以最新价成交
                    // entrustPrice < newestPrice
                    if (entrustPrice < newestPrice) {
                        return R.error("开仓失败,委托价格小于最新价");
                    }
                    break;

            }

            //每手交易单位(合约面值)
            Integer tradingUnit = 0;


            tradingUnit = setCompany(qh.getName());

            //设置最新价格
            //entity.setNewestPrice(qh.getLastPrice());


            //所需资金 = 委托价格 * 每手交易单位 * 委托数量(手数)
            BigDecimal totalAmount = algorithm.getFundsRequired(entity.getEntrustPrice(), BigDecimal.valueOf(entity.getEntrustNumber()), tradingUnit);


            //

            //所需资金:与前端传来参数对比 需要资金 = 委托价格 * 每手交易单位 * 委托数量(手数)
            if (entity.getFundingNeeded().compareTo(totalAmount) == 0) {
                //a.compareTo(b) == 0
                entity.setFundingNeeded(totalAmount);

                //浮动盈亏=（当天结算价—昨日结算价）* 委托数量 * 合约面值
                BigDecimal getFloatingPL = algorithm.getFloatingPL(qh.getLastPrice(), qh.getDayUp(), new BigDecimal(entity.getEntrustNumber()), tradingUnit);
                entity.setFloatingPl(getFloatingPL);

                //获取最后交易日
                String tradeTime = algorithm.getLastTradingDay(qh.getName());
                entity.setTradeTime(tradeTime);

                //pre_freezing_margin:预冻结保证金 = 外汇期货合约价格 * 合约面值 * 手数 * 保证金比例
                BigDecimal getPreFreezingMargin = algorithm.getPreFreezingMargin(qh.getLastPrice(), new BigDecimal(tradingUnit), qh.getName().substring(0, 8));
                entity.setPreFreezingMargin(getPreFreezingMargin);

                //pre_freezing_frozen:预冻结手续费 = 委托数量（手）* 费用标准
                BigDecimal getPreFreezingServiceCharge = algorithm.getPreFreezingServiceCharge(new BigDecimal(entity.getEntrustNumber()));
                entity.setPreFreezingFrozen(getPreFreezingServiceCharge);


                //frozen_price:冻结资金 = 预计冻结保证金+预冻结手续费
                BigDecimal frozen_price = getPreFreezingMargin.add(getPreFreezingServiceCharge);
                entity.setFrozenPrice(frozen_price);


                //获取当前期权账户可用资金
                QueryWrapper<CdUserOptionAccountEntity> queryWrapper2 = new QueryWrapper<CdUserOptionAccountEntity>();
                queryWrapper2.eq("user_id", entity.getUserId());
                //getBalance:获取到当前期权账户可用资金
                BigDecimal nowBalance = accountDao.selectOne(queryWrapper2).getBalance();


                //deduction:要扣除的金额 = 冻结资金 + 所需资金
                BigDecimal deduction = frozen_price.add(totalAmount);

                //modifyPrice:要修改的金额(需要扣除的金额 = 当前账户可用资金 - 要扣除的金额)
                BigDecimal modifyPrice = nowBalance.subtract(deduction);

                //判断当前可用余额是否足以扣除本次交易金额 >=
                double nowBalance1 = nowBalance.doubleValue();
                double deduction1 = deduction.doubleValue();
                if (nowBalance1 < deduction1) {
                    return R.error("可用资金不足，开仓失败");
                }

                //设置可交易数量 = 当前的委托数量
                entity.setTradableNumber(new BigDecimal(entity.getEntrustNumber()));

                //做修改账户金额操作
                UpdateWrapper<CdUserOptionAccountEntity> updateWrapper2 = new UpdateWrapper<CdUserOptionAccountEntity>();
                updateWrapper2.set("balance", modifyPrice);
                //修改占用保证金  getPreFreezingMargin
                updateWrapper2.set("deposit_balance", getPreFreezingMargin);
                updateWrapper2.eq("user_id", entity.getUserId());
                int updateAssets = accountDao.update(null, updateWrapper2);


                //添加期权账户流水记录
                CdBankStatementsEntity bankStatements2 = new CdBankStatementsEntity();
                bankStatements2.setType(2);//类型（1银行账户 2期货账户）
                bankStatements2.setCoinType("人民币");
                bankStatements2.setBeforeAmount(nowBalance);//改变前金额
                bankStatements2.setAmountMoney(deduction);//这次操作的金额
                bankStatements2.setAfterAmount(modifyPrice);//改变后的金额
                bankStatements2.setTransferType(2);//流水类型(1:转账 2:流水)
                bankStatements2.setCreateTime(getDate);
                bankStatements2.setUserId(entity.getUserId());
                bankStatements2.setRemark("开仓");
                int addBankStatements2 = bankStatementsDao.insert(bankStatements2);


                if (updateAssets <= 0 && addBankStatements2 <= 0) {
                    return R.error("开仓失败");
                }


                //设置当前时间
                entity.setCreateTime(getDate);
                //设置状态(0开仓,1平仓)
                entity.setStatus(0);
                //是否自动平仓（0否,1是）
                entity.setType(0);
                this.save(entity);

                //往调仓记录表添加开仓记录
                CdTransferWarehouseRecordEntity record = new CdTransferWarehouseRecordEntity();
                //开仓表id
                record.setCid(entity.getId());
                record.setUserId(entity.getUserId());
                record.setDirection(0);//买卖方向(0开仓 1平仓)
                record.setType(0);//委托类型(0市价 1限价)
                record.setNumber(entity.getTradableNumber());//委托手数


                //getMarketValue:市值 = 外汇期货合约价格 * 委托数量 * 合约面值
                //BigDecimal getMarketValue = algorithm.getMarketValue(getNowPrice, configure.getEntrustNumber(), number);

                record.setBalance(deduction);//市值>>>金额
                record.setCreateTime(getDate);
                transferWarehouseRecordDao.insert(record);


                return R.ok();
            }
            return R.error("所需资金对不上");


        } catch (Exception e) {
            e.printStackTrace();
            return R.error(e.getMessage());
        }
    }


    //外汇期货持仓情况
    public CdHoldPositionInfoResp myPosition(int userId) {

        try {
            QueryWrapper<CdFuturesConfigureEntity> wrapper = new QueryWrapper<>();
            wrapper.eq("user_id", userId);
            CdFuturesConfigureEntity entity = this.getOne(wrapper);


            //查询合约表信息
            QhscGangjiaosuoEntity qh = qhscGangjiaosuoDao.selectById(entity.getPid());
            entity.setEname(qh.getEName());
            entity.setName(qh.getName());

            CdHoldPositionInfoResp resp = new CdHoldPositionInfoResp();
            resp.setId(entity.getId());
            resp.setUserId(entity.getUserId());
            //交易方向(0空头 卖,1多头 买)
            if (entity.getTradingDirection() == 0) {
                resp.setTradingDirection("卖");
            } else if (entity.getTradingDirection() == 1) {
                resp.setTradingDirection("买");
            }

            resp.setEntrustPrice(entity.getEntrustPrice());
            resp.setEntrustNumber(entity.getEntrustNumber());
            //持仓状态(0.开仓 1.平仓)
            if (entity.getStatus() == 0) {
                resp.setStatus("开仓");
            } else if (entity.getStatus() == 1) {
                resp.setStatus("平仓");
            }

            resp.setCreateTime(entity.getCreateTime().substring(0, 10));
            resp.setUpdateTime(entity.getUpdateTime());
            resp.setTradeTime(entity.getTradeTime());
            resp.setName(entity.getName());
            resp.setEname(entity.getEname());
            resp.setPreFreezingMargin(entity.getPreFreezingMargin());


            //合约面值
            Integer tradingUnit = setCompany(entity.getName());

            //BigDecimal todayPrice, BigDecimal yesterdayPrice, BigDecimal commissionQuantity, Integer contractFaceValue
            //浮动盈亏=（当天结算价—昨日结算价）* 委托数量 * 合约面值(1份合约=一手，一手=？吨)
            BigDecimal getFloatingPL = algorithm.getFloatingPL(qh.getLastPrice(), qh.getDayUp(), BigDecimal.valueOf(tradingUnit), entity.getEntrustNumber());
            resp.setFloatingPl(getFloatingPL);


            return resp;
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }

    }


    /**
     * 平仓操作
     * 需求:平仓前判断行情滚动时间是不等于最后交易日,行情滚动是最后交易日当天才能滚动
     * 平仓，选择当天平台的日期，已当天的行情价格，卖出或者买入相同的期货合约平仓。
     * 外汇期货盈亏=（平仓时的外汇汇率价格-下单时外汇汇率价格) * 委托数量 * 合约面值。
     *
     * @param id
     * @return
     */
    @Override
    public R buyIn(int id, String date, Integer userId) {
        try {

            String getDate = template.opsForValue().get("date:userId_" + userId);
            //根据id查询开仓后的信息
            CdFuturesConfigureEntity configure = baseMapper.selectById(id);

            //判断传的行情滚动日期是否等于最后交易日
            /*if (!date.trim().equals(configure.getTradeTime().substring(0, 10))) {
                return R.error("行情滚动时间要与最后交易日一致");
            }*/


            //根据合约id表查询合约信息
            QhscGangjiaosuoEntity qh = qhscGangjiaosuoDao.selectById(configure.getPid());
            //获取该合约的代码
            String ename = qh.getEName();
            //获取每一手的单位
            Integer number = setCompany(qh.getName());

            //根据合约代码查询平仓最后交易日那天的的汇率信息
            QueryWrapper<QhscGangjiaosuoEntity> queryWrapper2 = new QueryWrapper<QhscGangjiaosuoEntity>();
            queryWrapper2.eq("e_name", ename);
            queryWrapper2.eq("date", date);

            QhscGangjiaosuoEntity now = qhscGangjiaosuoDao.selectOne(queryWrapper2);
            if (now == null) {
                //查询不到合约表当天的信息时候弹出
                return R.error("不在交易时段内，无法进行交易");
            }

            //查询合约当天外汇汇率价格
            double getNowPrice = now.getLastPrice().doubleValue();

            //查询开仓时候的委托价格
            double entrustPrice = configure.getEntrustPrice().doubleValue();

            /**
             * 期货的平仓成交规则：
             * configure.getTradingDirection() : 交易方向(0空头,1多头)
             * 空方情况下，可交易数量为空方合约持仓总量，委托价格大于等于最新价，以最新价成交。
             * 多方情况下，可交易数量为多方合约持仓总量，委托价格小于等于最新价，以最新价成交；
             */
            //TODO 暂时注释
           /* switch (configure.getTradingDirection()) {

                case 0:
                    //空方情况下:委托价格大于等于最新价，以最新价成交。
                    if (entrustPrice < getNowPrice) {
                        return R.error("平仓失败,委托价格小于最新价");
                    }
                    break;
                case 1:
                    //多方情况下:委托价格小于等于最新价，以最新价成交；
                    if (entrustPrice > getNowPrice) {
                        return R.error("平仓失败,委托价格大于最新价");
                    }
                    break;

            }*/


            //外汇期货盈亏=（平仓时的外汇汇率价格(等同于当天合约的最新价)-下单时外汇汇率价格(开仓时候的委托价)) * 委托数量 * 合约面值。
            BigDecimal pl = algorithm.get_wh_pl(BigDecimal.valueOf(getNowPrice), configure.getEntrustPrice(), new BigDecimal(configure.getEntrustNumber()), new BigDecimal(number));

            //getMarketValue:市值 = 外汇期货合约价格 * 委托数量 * 合约面值
            BigDecimal getMarketValue = algorithm.getMarketValue(BigDecimal.valueOf(getNowPrice), configure.getEntrustNumber(), number);

            CdFuturesConfigureEntity entity = new CdFuturesConfigureEntity();
            //设置平仓时间
            entity.setUpdateTime(date);
            //设置状态(0.开仓 1平仓)
            entity.setStatus(1);
            //设置平仓价格:市场当前价
            entity.setCloseRate(BigDecimal.valueOf(getNowPrice));
            //设置盈亏
            entity.setFloatingPl(pl);
            entity.setId(id);

            //平仓时 修改冻结资金为0
            BigDecimal frozenPrice = configure.getFrozenPrice();
            entity.setFrozenPrice(new BigDecimal(0));

            int update = baseMapper.updateById(entity);


            //获取当前期权账户可用资金
            QueryWrapper<CdUserOptionAccountEntity> queryWrapper3 = new QueryWrapper<CdUserOptionAccountEntity>();
            queryWrapper3.eq("user_id", configure.getUserId());
            //nowBalance:获取到当前期权账户可用资金
            BigDecimal nowBalance = accountDao.selectOne(queryWrapper3).getBalance();

            //改变后的金额 再加上冻结的资金
            BigDecimal newPrice = nowBalance.add(getMarketValue).add(frozenPrice);


            //做修改账户金额操作
            UpdateWrapper<CdUserOptionAccountEntity> modify = new UpdateWrapper<CdUserOptionAccountEntity>();
            //平仓后加金额 = 当前可用余额 + 市值
            modify.set("balance", newPrice);
            modify.eq("user_id", configure.getUserId());
            int updateAssets = accountDao.update(null, modify);


            //添加期权账户流水记录
            CdBankStatementsEntity bankStatements2 = new CdBankStatementsEntity();
            bankStatements2.setType(2);//类型（1银行账户 2期货账户）
            bankStatements2.setCoinType("人民币");
            bankStatements2.setBeforeAmount(nowBalance);//改变前金额
            bankStatements2.setAmountMoney(getMarketValue);//这次操作的金额
            bankStatements2.setAfterAmount(newPrice);//改变后的金额
            bankStatements2.setTransferType(2);//流水类型(1:转账 2:流水)
            bankStatements2.setCreateTime(configure.getTradeTime());
            bankStatements2.setRemark("平仓");
            bankStatements2.setUserId(configure.getUserId());
            int addBankStatements2 = bankStatementsDao.insert(bankStatements2);


            //往调仓记录表添加平仓记录
            CdTransferWarehouseRecordEntity record = new CdTransferWarehouseRecordEntity();
            //开仓表id
            record.setCid(configure.getId());
            record.setUserId(configure.getUserId());
            record.setDirection(1);//买卖方向(0开仓 1平仓)
            record.setType(0);//委托类型(0市价 1限价)
            record.setNumber(configure.getTradableNumber());//委托手数
            record.setBalance(getMarketValue);//市值>>>金额
            record.setCreateTime(configure.getTradeTime());
            int addTransferWarehouseRecor = transferWarehouseRecordDao.insert(record);


            if (addTransferWarehouseRecor <= 0 && updateAssets <= 0 && addBankStatements2 <= 0 && update <= 0) {
                return R.error("平仓失败");
            }
            return R.ok();
        } catch (Exception e) {
            e.printStackTrace();
            return R.error(e.getMessage());
        }
    }


    //调仓前展示详情
    /*@Override
    public CdTransferWarehouseDetailResp transferWarehouseByDetail(int id) {

        CdFuturesConfigureEntity futuresConfigure = baseMapper.selectById(id);

        //python表:根据合约id查询合约相关数据
        QhscGangjiaosuoEntity qh = qhscGangjiaosuoDao.selectById(futuresConfigure.getPid());
        futuresConfigure.setName(qh.getName());
        futuresConfigure.setEname(qh.getEName());
        futuresConfigure.setLastPrice(qh.getLastPrice());
        futuresConfigure.setU(qh.getU());
        futuresConfigure.setDayUp(qh.getDayUp());


        String have = qh.getHava();
        BigDecimal getHave = new BigDecimal(0);
        if (have.contains("万")) {
            have = have.substring(0, have.length() - 1);
            double getHaveByDouble = Double.valueOf(have);

            getHave = MoneyConversionUtil.moneyConversion(getHaveByDouble);

        }
        //用户选择的买卖方向为开仓时候:查python的持仓总量
        futuresConfigure.setTradableQuantityByBuy(getHave);
        //选择为平仓时候:查自己的持仓总量
        futuresConfigure.setTradableQuantityBySell(futuresConfigure.getTradableNumber());


        //获取当前期权账户可用资金
        QueryWrapper<CdUserOptionAccountEntity> queryWrapper3 = new QueryWrapper<CdUserOptionAccountEntity>();
        queryWrapper3.eq("user_id", futuresConfigure.getUserId());
        //nowBalance:获取到当前期权账户可用资金
        BigDecimal nowBalance = accountDao.selectOne(queryWrapper3).getBalance();
        futuresConfigure.setAvailableFunds(nowBalance);
        return futuresConfigure;
    }
*/

    public static void main(String[] args) {
        String str = "2020-10-30 00:00:00";
        System.out.println(str.substring(0, 10));
    }

    //调仓前展示详情
    @Override
    public CdTransferWarehouseDetailResp transferWarehouseByDetail(int id) {
        try {
            CdFuturesConfigureEntity configure = baseMapper.selectById(id);
            CdTransferWarehouseDetailResp resp = new CdTransferWarehouseDetailResp();
            //python表:根据合约id查询合约相关数据
            QhscGangjiaosuoEntity qh = qhscGangjiaosuoDao.selectById(configure.getPid());

            String have = qh.getHava();
            BigDecimal getHave = new BigDecimal(0);
            if (have.contains("万")) {
                have = have.substring(0, have.length() - 1);
                double getHaveByDouble = Double.valueOf(have);

                getHave = MoneyConversionUtil.moneyConversion(getHaveByDouble);

            }
            getHave = new BigDecimal(have);


            //获取当前期权账户可用资金
            QueryWrapper<CdUserOptionAccountEntity> queryWrapper = new QueryWrapper<CdUserOptionAccountEntity>();
            queryWrapper.eq("user_id", configure.getUserId());
            //nowBalance:获取到当前期权账户可用资金
            BigDecimal nowBalance = accountDao.selectOne(queryWrapper).getBalance();


            //id
            resp.setId(configure.getId());
            //交易方向
            resp.setTradingDirection(configure.getTradingDirection());
            //合约主键
            resp.setPid(configure.getPid());
            //开仓委托价
            resp.setNewestPrice(configure.getNewestPrice());
            //名称
            resp.setName(qh.getName());
            //代码
            resp.setEname(configure.getEname());
            //市价(最新价)
            resp.setLastPrice(qh.getLastPrice());
            //涨跌幅
            resp.setU(qh.getU());
            //昨结
            resp.setDayUp(qh.getDayUp());
            //买卖方向为开仓时候:可交易数量为Python表的持仓总量
            resp.setTradableQuantityByBuy(getHave);
            //买卖方向为平仓时候:可交易数量为自己的持仓总量
            resp.setTradableQuantityBySell(configure.getTradableNumber());
            //当前期权账户可用资金
            resp.setAvailableFunds(nowBalance);
            //买入价
            resp.setBuyPrice(qh.getBuyPrice());
            //卖出价
            resp.setCellPrice(qh.getCellPrice());
            //买量
            resp.setBuy(qh.getBuy());
            //卖量
            resp.setCell(qh.getCell());
            //总量
            resp.setSuoyou(qh.getSuoyou());
            //代码
            resp.setEname(qh.getEName());
            //涨跌额
            resp.setUdPrice(qh.getUdPrice());


            return resp;
        } catch (NumberFormatException e) {
            e.printStackTrace();
            return null;
        }
    }

    //外汇期货平仓绩效
    @Override
    public CdFuturesConfigureEntityVo buyInAchievements(int id, Integer userId) {
        String getDate = template.opsForValue().get("date:userId_" + userId);
        Integer getPid = baseMapper.selectById(id).getPid();

        //每手交易单位
        Integer tradingUnit = 0;
        //python表:根据合约id查询合约相关数据
        QhscGangjiaosuoEntity qh = qhscGangjiaosuoDao.selectById(getPid);

        tradingUnit = setCompany(qh.getName());


        //期货开仓平仓记录
        CdFuturesConfigureEntity entity = baseMapper.selectById(id);


        //开仓后资产市值：开仓价格 * 委托数量 * 合约面值
        BigDecimal marketValueByOpenPosition = algorithm.getMarketValue(entity.getEntrustPrice(), entity.getEntrustNumber(), tradingUnit);

        //平仓后资产市值：平仓价格 * 委托数量 * 合约面值
        BigDecimal marketValueByClosePosition = algorithm.getMarketValue(entity.getCloseRate(), entity.getEntrustNumber(), tradingUnit);


        CdFuturesConfigureEntityVo vo = new CdFuturesConfigureEntityVo();
        vo.setId(entity.getId());
        //代码
        vo.setEName(qh.getEName());
        vo.setName(qh.getName());

        if (entity.getTradingDirection() == 0) {
            vo.setTradingDirection("做空");
        } else if (entity.getTradingDirection() == 1) {
            vo.setTradingDirection("做多");
        }


        //资产市值1
        vo.setMarketValue1(marketValueByOpenPosition);
        //资产市值2
        vo.setMarketValue2(marketValueByClosePosition);

        //开仓时间
        vo.setCreateTime(DateUtils.dateForByStr(entity.getCreateTime()));

        //平仓时间
        vo.setUpdateTime(DateUtils.dateForByStr(entity.getUpdateTime()));

        //数量
        vo.setEntrustNumber(entity.getEntrustNumber());

        //外汇盈亏=（平仓时的外汇汇率价格 - 下单时外汇汇率价格) * 委托数量 * 合约面值。
        BigDecimal pl = algorithm.get_wh_pl(entity.getCloseRate(), entity.getNewestPrice(), BigDecimal.valueOf(entity.getEntrustNumber()), BigDecimal.valueOf(tradingUnit));

        vo.setWh_pl(pl);
        vo.setCloseRate(entity.getCloseRate());
        vo.setEntrustPrice(entity.getEntrustPrice());


        return vo;
    }


    //外汇现货盈亏
    @Override
    public CdBeforeHedgingResp beforeHedging(int userId) {

        //根据用户id查询当前的pid 以及 开仓平仓时间
        QueryWrapper<CdFuturesConfigureEntity> wrapper = new QueryWrapper<CdFuturesConfigureEntity>();
        wrapper.eq("user_id", userId);
        CdFuturesConfigureEntity configureEntity = baseMapper.selectOne(wrapper);


        //根据pid查询合约名称
        QhscGangjiaosuoEntity qh = qhscGangjiaosuoDao.selectById(configureEntity.getPid());


        //合约名称
        String name = qh.getName().substring(0, 2);


        name += "人民币中间价";

        //获取代码
        QueryWrapper<WhscRenminbipinzhongEntity> wrapper1 = new QueryWrapper<WhscRenminbipinzhongEntity>();
        wrapper1.like("name", name).last("limit 0,1");
        WhscRenminbipinzhongEntity whsc = renminbipinzhongDao.selectOne(wrapper1);


        //每手交易单位
        Integer tradingUnit = setCompany(qh.getName().substring(0, 2) + "兑人民币期货");

        CdBeforeHedgingResp resp = new CdBeforeHedgingResp();

        //开仓交易时间
        String start = DateUtils.dateForByStr(configureEntity.getCreateTime());
        //平仓交易时间
        String end = DateUtils.dateForByStr(configureEntity.getUpdateTime());

        //获取开仓时间当天的现货价格
        QueryWrapper<WhscRenminbipinzhongEntity> getDateWapper = new QueryWrapper<WhscRenminbipinzhongEntity>();
        getDateWapper.eq("name", name);
        getDateWapper.eq("date", start);
        WhscRenminbipinzhongEntity getDate1 = renminbipinzhongDao.selectOne(getDateWapper);

        //获取平仓时间当天的现货价格
        QueryWrapper<WhscRenminbipinzhongEntity> getDateWapper1 = new QueryWrapper<WhscRenminbipinzhongEntity>();
        getDateWapper1.eq("name", name);
        getDateWapper1.eq("date", end);
        WhscRenminbipinzhongEntity getDate2 = renminbipinzhongDao.selectOne(getDateWapper1);


        //开仓金额(外币) = 汇率 * 合约面值
        BigDecimal foreignCurrencyByOpen = algorithm.getForeignCurrency(BigDecimal.valueOf(getDate1.getLastPrice()), new BigDecimal(10000000));

        //金额(外币) = 汇率价格 * 合约面值
        BigDecimal foreignCurrencyByClose = algorithm.getForeignCurrency(BigDecimal.valueOf(getDate2.getLastPrice()), new BigDecimal(10000000));


        //盈亏
        BigDecimal profitLoss = new BigDecimal(0).setScale(2, BigDecimal.ROUND_HALF_UP);

        profitLoss = foreignCurrencyByOpen.subtract(foreignCurrencyByClose);


        //交易方向(0空头 卖,1多头 买)
        if (configureEntity.getTradingDirection() == 0) {
            resp.setTradingDirection("卖");
        } else if (configureEntity.getTradingDirection() == 1) {
            resp.setTradingDirection("买");
        }
        resp.setProfitLoss(profitLoss);
        resp.setName(qh.getName().substring(0, 6));


        //金额外币(开仓)
        resp.setForeignCurrencyByOpen(new BigDecimal(10000000));
        //金额外币(平仓)
        resp.setForeignCurrencyByClose(new BigDecimal(10000000));
/*
        //开仓所需资金
        BigDecimal openNeed = configureEntity.getEntrustPrice().multiply(foreignCurrencyByOpen);
        //平仓所需资金
        BigDecimal closeNeed = configureEntity.getCloseRate().multiply(foreignCurrencyByClose);*/

        //开仓所需资金
        BigDecimal openNeed = new BigDecimal(getDate1.getLastPrice()).multiply(foreignCurrencyByOpen);
        //平仓所需资金
        BigDecimal closeNeed = new BigDecimal(getDate2.getLastPrice()).multiply(foreignCurrencyByClose);

        resp.setNeededPriceByOpen(openNeed);
        resp.setNeededPriceByClose(closeNeed);
        resp.setCreateTime(start);
        resp.setUpdateTime(end);
        resp.setExchangeRate1(new BigDecimal(getDate1.getLastPrice()));
        resp.setExchangeRate2(new BigDecimal(getDate2.getLastPrice()));
        return resp;
    }

    //对冲后损益情况
    @Override
    public CdAfterHedgingResp afterHedging(int userId) {
        try {
            CdAfterHedgingResp resp = new CdAfterHedgingResp();

            QueryWrapper<CdFuturesConfigureEntity> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("user_id", userId);
            //查询当前开仓信息
            CdFuturesConfigureEntity config = baseMapper.selectOne(queryWrapper);

            //开仓时间
            String startTime = DateUtils.dateForByStr(config.getCreateTime());
            //样本区间截止时间：开仓时间后三个月
            String endTime = DateUtils.getAfterMonth(startTime, 3);


            QhscGangjiaosuoEntity qhEntity = qhscGangjiaosuoDao.selectById(config.getPid());


            //根据开仓时间-开仓时间后三个月查询
            QueryWrapper<QhscGangjiaosuoEntity> qhWarapper = new QueryWrapper<>();
            qhWarapper.eq("e_name", qhEntity.getEName());
            qhWarapper.ge("date", startTime).le("date", endTime);

            List<QhscGangjiaosuoEntity> rmb = qhscGangjiaosuoDao.selectList(qhWarapper);
            String code = "";
            String name = "";

            for (QhscGangjiaosuoEntity e : rmb) {

                String s = String.valueOf(e.getLastPrice());
                code += s + ",";
                name = e.getName();
            }
            code = code.substring(0, code.length() - 1);


            //每手交易单位(合约面值)
            Integer tradingUnit = setCompany(name);

            //confidenceLevel   置信水平
            String confidenceLevel = "0.95";
            //validity          持有效
            String validity = "90";
            //实验次数
            String experimentsNumber = "1000";


            //Z是传入进来的下单数*期货合约面值
            Integer Z = config.getEntrustNumber() * tradingUnit;


            String afterBefore = "";
            String afterURL = "";
            String beforeURL = "";
            do {
                //I为次数，t是天数，Z是传入进来的下单数*期货合约面值，arf为置信水平，
                afterBefore = PyUtil.getCodeByPhoto(PyUtil.path, PyUtil.beforeAndAfter, experimentsNumber, validity, String.valueOf(Z), confidenceLevel, code);

                if (afterBefore.length() > 0) {

                    String[] split = afterBefore.split(",");
                    String p1 = split[0];
                    String p2 = split[1];
                    afterURL = p2;
                    beforeURL = p1;
                }

            } while (afterBefore.isEmpty());


            resp.setBeforeHedgingUrl(beforeURL);
            resp.setAfterHedgingUrl(afterURL);
            resp.setName(name.substring(0, 6));
            resp.setStartTime(startTime);
            resp.setEndTime(endTime);
            resp.setEffective(validity);
            resp.setConfidenceLevel(confidenceLevel);
            resp.setExperimentNum(experimentsNumber);


            return resp;
        } catch (IOException e) {
            e.printStackTrace();
            log.error(e.getMessage());
        }


        return null;
    }

    @Override
    public boolean delOpenPosition(int id, int userId) {
        //删除开仓信息
        int flag = baseMapper.deleteById(id);
        if (flag == 1) {
            //先查看流水表要返回的金额

            QueryWrapper<CdBankStatementsEntity> wrapper = new QueryWrapper<>();
            wrapper.eq("user_id", userId);
            wrapper.eq("transfer_type", "2");

            CdBankStatementsEntity bankStatementsEntity = bankStatementsDao.selectOne(wrapper);


            //删除账户流水表记录
            Map<String, Object> map = new HashMap<>();
            //删除类型为流水的 : (1:转账 2:流水)
            map.put("transfer_type", "2");
            map.put("user_id", userId);

            bankStatementsDao.deleteByMap(map);


            //头寸表记录
            //开仓平仓调仓记录表
            //修改用户账户今天为原来的

        }

        return false;
    }


    //设置交易单位(合约面值)
    public static Integer setCompany(String getName) {
        try {
            //每手交易单位
            Integer tradingUnit = 0;
            //设置每手交易单位
            switch (getName.substring(0, 8)) {
                case algorithm.AUDAndRMB:
                    tradingUnit = algorithm.AUDExchangeRMB;
                    break;
                case algorithm.EURAndRMB:
                    tradingUnit = algorithm.EURExchangeRMB;
                    break;
                case algorithm.JPYAndRMB:
                    tradingUnit = algorithm.JPYExchangeRMB;
                    break;
                case algorithm.RMBAndUSD:
                    tradingUnit = algorithm.RMBExchangeUSD;
                    break;
                case algorithm.USDAndRMB:
                    tradingUnit = algorithm.USDExchangeRMB;
                    break;

            }
            return tradingUnit;
        } catch (Exception e) {
            e.printStackTrace();
            return -1;
        }
    }


}



