package com.msdw.tms.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.msdw.tms.common.utils.R;
import com.msdw.tms.common.utils.trading_rules.algorithm;
import com.msdw.tms.dao.*;
import com.msdw.tms.entity.*;
import com.msdw.tms.service.CdTransferWarehouseRecordService;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;

/**
 * @描述：调仓记录表 服务类
 * @作者: Rong
 * @日期: 2020-10-10
 */
@Service("cdTransferWarehouseRecordService")
public class CdTransferWarehouseRecordServiceImpl extends ServiceImpl<CdTransferWarehouseRecordDao, CdTransferWarehouseRecordEntity> implements CdTransferWarehouseRecordService {

    @Resource
    private CdUserOptionAccountDao accountDao;
    @Resource
    private CdFuturesConfigureDao futuresConfigureDao;
    @Resource
    public QhscGangjiaosuoDao qhscGangjiaosuoDao;

    @Resource
    public CdBankStatementsDao bankStatementsDao;

    @Resource
    public CdTransferWarehouseRecordDao transferWarehouseRecordDao;
    @Resource
    public RedisTemplate<String, String> template;

    /**
     * 开仓买入 平仓卖出
     * <p>
     * 多头下单后，可以选择调仓
     * 调仓成功的前提是期货期权账号可用资金>=本次调仓所需要的资金，调仓下单成功，否则，提示可用资金不足，调仓失败。(完成)
     * <p>
     * <p>
     * <p>
     * 2.如果减仓（减仓的数量不超过持仓中的多头数量），扣除保证金，按照最终的委托数量核算市值，浮动盈亏（同上公式）。
     * 市值=外汇期货合约价格*（原持仓量- 本次调仓量）*合约面值，浮动盈亏=（当天结算价—昨日结算价）*（原持仓量-本次调仓量）*合约面值。
     * <p>
     * <p>
     * 空头下单后，可以选择调仓，如果加仓，按照多头下单，扣除保证金，按照最终的委托数
     * 量核算市值，浮动盈亏。如果减仓（减仓的数量不超过持仓中的空头数量），扣除保证金，
     * 按照最终的委托数量核算市值，浮动盈亏（同上公式）。
     */

    //调仓操作
    @Override
    public R add(CdFuturesConfigureEntity entity) {

        try {
            String getDate = template.opsForValue().get("date:userId_" + entity.getUserId());

            /**可交易数量(用户选择的买卖方向为开仓时候:查python表的持仓总量/用户选择的买卖方向为平仓时候:查自己的持仓总量)*/
            CdFuturesConfigureEntity old = futuresConfigureDao.selectById(entity.getId());
            QhscGangjiaosuoEntity qh = qhscGangjiaosuoDao.selectById(old.getPid());
            //查python的持仓总量
            BigDecimal getPyNumber = new BigDecimal(qh.getHava());
            //自己的持仓总量
            BigDecimal getMeNumber = old.getTradableNumber();


            //根据合约代码查询当天的汇率信息
            QueryWrapper<QhscGangjiaosuoEntity> queryWrapper2 = new QueryWrapper<QhscGangjiaosuoEntity>();
            queryWrapper2.eq("e_name", qh.getEName());
            //TODO 调仓时间为选中的时间
            queryWrapper2.eq("date", getDate);
            QhscGangjiaosuoEntity getNow = qhscGangjiaosuoDao.selectOne(queryWrapper2);
            if (getNow == null) {
                return R.error("暂时没有" + getDate + "的外汇信息");
            }
            //当前调仓输入的委托价
            double entrustPrice = entity.getEntrustPrice().doubleValue();

            //查询合约当天外汇汇率价格
            double getNowPrice = getNow.getLastPrice().doubleValue();


            //根据合约名称设置每手交易单位(合约面值)
            Integer tradingUnit = CdFuturesConfigureServiceImpl.setCompany(qh.getName());

            //获取当前期权账户可用资金
            QueryWrapper<CdUserOptionAccountEntity> getPrice = new QueryWrapper<CdUserOptionAccountEntity>();
            getPrice.eq("user_id", entity.getUserId());
            //nowBalance:获取到当前期权账户可用资金
            BigDecimal nowBalance = accountDao.selectOne(getPrice).getBalance();


            //判断用户选择的是平仓还是开仓 (0开仓 1平仓)
            switch (entity.getStatus()) {
                case 0:
                    //0开仓表买入：用户选择的买卖方向为开仓时候:获取当前合约的持仓总量


                    //市值:外汇期货合约价格*（原持仓量 + 本次调仓量）* 合约面值
                    //BigDecimal getMarketValueByAddition = algorithm.getMarketValueByAddition(now.getLastPrice(), getMeNumber, BigDecimal.valueOf(entity.getEntrustNumber()), CdFuturesConfigureServiceImpl.setCompany(qh.getName()));

                    //扣除保证金(保证金 = 外汇期货合约价格 * 委托数量 * 合约面值 * 保证金比例)
                    BigDecimal bzj = algorithm.freezingMargin(getNow.getLastPrice(), entity.getTradableNumber(), new BigDecimal(tradingUnit), qh.getName().substring(0, 8));


                    //TODO 总共要扣除的金额 =  需要扣除的保证金 + 所需资金

                    //所需资金 = 委托价格  * 委托数量(手数) * 每手交易单位
                    BigDecimal funding_needed = algorithm.getFundsRequired(entity.getEntrustPrice(), BigDecimal.valueOf(entity.getEntrustNumber()), tradingUnit);


                    /**
                     * 期货的开仓成交规则：
                     * 多方情况下，委托价格大于等于最新价，以最新价成交；
                     * 空方情况下，委托价格小于等于最新价，以最新价成交。
                     */
                    // tODO 暂时注释
                    //交易方向(0空头,1多头)
                    switch (old.getTradingDirection()) {
                        case 0:
                            //空方情况下:委托价格小于等于最新价，以最新价成交
                            // entrustPrice > newestPrice
                            if (entrustPrice > getNowPrice) {
                                return R.error("开仓失败,委托价格大于最新价");
                            }
                            break;
                        case 1:
                            //多方情况下:委托价格大于等于最新价，以最新价成交
                            // entrustPrice < newestPrice
                            if (entrustPrice < getNowPrice) {
                                return R.error("开仓失败,委托价格小于最新价");
                            }
                            break;

                    }
                    //对当前头寸表 需要修改的操作:
                    // 交易方向?、
                    CdFuturesConfigureEntity modifyByCondition = new CdFuturesConfigureEntity();

                    //最新价格:开仓时候 最新价格以当天汇率价格
                    modifyByCondition.setNewestPrice(getNow.getLastPrice());

                    //委托价格 当前输入的价格
                    modifyByCondition.setEntrustPrice(entity.getEntrustPrice());

                    //委托数量
                    modifyByCondition.setEntrustNumber(entity.getEntrustNumber());

                    //预冻结保证金 = 外汇期货合约价格 * 合约面值 * 手数 * 保证金比例
                    BigDecimal getPreFreezingMargin = algorithm.getPreFreezingMargin(qh.getLastPrice(), new BigDecimal(tradingUnit), qh.getName().substring(0, 8));
                    modifyByCondition.setPreFreezingMargin(getPreFreezingMargin);

                    //预冻结手续费 = 委托数量（手）* 费用标准
                    BigDecimal getPreFreezingServiceCharge = algorithm.getPreFreezingServiceCharge(new BigDecimal(entity.getEntrustNumber()));
                    modifyByCondition.setPreFreezingFrozen(getPreFreezingServiceCharge);


                    //frozen_price:冻结资金 = 预计冻结保证金 + 预冻结手续费
                    BigDecimal frozen_price = getPreFreezingMargin.add(getPreFreezingServiceCharge);
                    modifyByCondition.setFrozenPrice(frozen_price);

                    //设置所需资金
                    modifyByCondition.setFundingNeeded(funding_needed);


                    //计算合计操作的总持仓量 = 自己的持仓总量 + 本次要加仓的数量
                    BigDecimal getTotal = getMeNumber.add(BigDecimal.valueOf(entity.getEntrustNumber()));
                    //可交易数量 = 之前的委托数量 + 本次委托数量
                    modifyByCondition.setTradableNumber(getTotal);

                    //委托数量
                    modifyByCondition.setEntrustNumber(getTotal.intValue());
                    // 浮动盈亏=（当天结算价—昨日结算价）* 委托数量 * 合约面值(1份合约=一手，一手=？吨)
                    BigDecimal floatPl = algorithm.getFloatingPL(getNow.getLastPrice(), getNow.getDayUp(), new BigDecimal(entity.getEntrustNumber()), CdFuturesConfigureServiceImpl.setCompany(qh.getName()));
                    //设置浮动盈亏
                    modifyByCondition.setFloatingPl(floatPl);
                    //设置为开仓
                    modifyByCondition.setStatus(0);

                    //总共要扣除的金额 =  需要扣除的保证金 + 所需资金
                    BigDecimal TotalAmount = funding_needed.add(bzj);
                    //判断当前可用余额是否足以扣除本次交易金额
                    if (nowBalance.doubleValue() < TotalAmount.doubleValue()) {
                        return R.error("可用资金不足，开仓失败");
                    }

                    //开仓时间
                    modifyByCondition.setCreateTime(getDate);
                    modifyByCondition.setId(entity.getId());
                    int modify = futuresConfigureDao.updateById(modifyByCondition);


                    //modifyPrice:要做修改的金额(要做修改的金额 = 当前账户可用资金 - 要扣除的金额)
                    BigDecimal modifyPrice = nowBalance.subtract(TotalAmount);


                    //对用户期权账户修改金额操作
                    UpdateWrapper<CdUserOptionAccountEntity> updateWrapper2 = new UpdateWrapper<CdUserOptionAccountEntity>();
                    updateWrapper2.set("balance", modifyPrice);
                    updateWrapper2.eq("user_id", entity.getUserId());
                    int updateAssets = accountDao.update(null, updateWrapper2);


                    //添加期权账户流水记录
                    CdBankStatementsEntity bankStatements2 = new CdBankStatementsEntity();
                    bankStatements2.setType(2);//类型（1银行账户 2期货账户）
                    bankStatements2.setCoinType("人民币");
                    bankStatements2.setBeforeAmount(nowBalance);//改变前金额
                    bankStatements2.setAmountMoney(TotalAmount);//这次操作的金额
                    bankStatements2.setAfterAmount(modifyPrice);//改变后的金额
                    bankStatements2.setTransferType(2);//流水类型(1:转账 2:流水)
                    bankStatements2.setCreateTime(getDate);
                    bankStatements2.setUserId(entity.getUserId());
                    bankStatements2.setRemark("开仓");
                    int addBankStatements2 = bankStatementsDao.insert(bankStatements2);


                    //往开仓表添加开仓记录
                    //往流水记录表添加开仓流水记录
                    CdTransferWarehouseRecordEntity record = new CdTransferWarehouseRecordEntity();
                    //开仓表id
                    record.setCid(entity.getId());
                    record.setUserId(entity.getUserId());
                    record.setDirection(0);//买卖方向(0开仓 1平仓)
                    record.setType(0);//委托类型(0市价 1限价)
                    record.setNumber(entity.getTradableNumber());//委托手数
                    record.setBalance(TotalAmount);//市值>>>金额
                    record.setCreateTime(getDate);//TODO 调仓时开仓时间为选中的时间
                    int addRe = transferWarehouseRecordDao.insert(record);

                    if (modify <= 0 && updateAssets <= 0 && addBankStatements2 <= 0 && addRe <= 0) {
                        return R.error("调仓失败");
                    }

                    break;

                case 1:
                    /* 1平仓表示卖出 */

                    Integer num = entity.getEntrustNumber();

                    //当前可交易数量 - 我要平仓的数据
                    Integer getTradableNumber = old.getTradableNumber().intValue();
                    if (getTradableNumber - num < 1) {
                        return R.error("委托数量不能是持仓总量,至少留一手~");
                    }


                    /**
                     * 期货的平仓成交规则：
                     * configure.getTradingDirection() : 交易方向(0空头,1多头)
                     * 空方情况下，可交易数量为空方合约持仓总量，委托价格大于等于最新价，以最新价成交。
                     * 多方情况下，可交易数量为多方合约持仓总量，委托价格小于等于最新价，以最新价成交；
                     */
                    // TODO 暂时注释
                    switch (old.getTradingDirection()) {
                        case 0:
                            //空方情况下:委托价格大于等于最新价，以最新价成交。
                            if (entrustPrice < getNowPrice) {
                                return R.error("平仓失败,委托价格小于最新价");
                            }
                            break;
                        case 1:
                            //多方情况下:委托价格小于等于最新价，以最新价成交；
                            if (entrustPrice > getNowPrice) {
                                return R.error("平仓失败,委托价格大于最新价");
                            }
                            break;

                    }


                    //外汇期货盈亏=（平仓时的外汇汇率价格(等同于当天合约的最新价)-下单时外汇汇率价格(开仓时候的委托价)) * 委托数量 * 合约面值。
                    BigDecimal pl = algorithm.get_wh_pl(BigDecimal.valueOf(getNowPrice), old.getEntrustPrice(), new BigDecimal(old.getEntrustNumber()), new BigDecimal(tradingUnit));

                    //getMarketValue:市值 = 外汇期货合约价格 * 委托数量 * 合约面值
                    BigDecimal getMarketValue = algorithm.getMarketValue(BigDecimal.valueOf(getNowPrice), entity.getEntrustNumber(), tradingUnit);


                    //根据id查询开仓后的信息
                    //CdFuturesConfigureEntity configure = futuresConfigureDao.selectById(entity.getId());


                    //对当前头寸表 需要修改的操作:
                    // 交易方向?、
                    CdFuturesConfigureEntity modifyByCondition1 = new CdFuturesConfigureEntity();

                    //设置平仓时间
                    modifyByCondition1.setUpdateTime(old.getTradeTime());
                    //设置状态(0.开仓 1平仓)
                    modifyByCondition1.setStatus(1);
                    //设置平仓价格:市场当前价
                    modifyByCondition1.setCloseRate(getMarketValue);
                    //设置盈亏
                    modifyByCondition1.setFloatingPl(pl);

                    modifyByCondition1.setId(entity.getId());
                    //修改设置可交易量
                    modifyByCondition1.setTradableNumber(BigDecimal.valueOf(old.getEntrustNumber()).subtract(BigDecimal.valueOf(entity.getEntrustNumber())));

                    //平仓持仓量 等于持仓量减去本次下单手数
                    modifyByCondition1.setEntrustNumber(old.getEntrustNumber() - entity.getEntrustNumber());

                    int modify1 = futuresConfigureDao.updateById(modifyByCondition1);


                    //改变后的金额
                    BigDecimal newPrice = nowBalance.add(getMarketValue);


                    //做修改账户金额操作
                    UpdateWrapper<CdUserOptionAccountEntity> modifyBypc = new UpdateWrapper<CdUserOptionAccountEntity>();
                    //平仓后加金额 = 当前可用余额 + 市值
                    modifyBypc.set("balance", newPrice);
                    modifyBypc.eq("user_id", entity.getUserId());
                    int updateAsset = accountDao.update(null, modifyBypc);


                    //往流水记录表添加平仓流水记录
                    CdTransferWarehouseRecordEntity record1 = new CdTransferWarehouseRecordEntity();
                    //开仓表id
                    record1.setCid(entity.getId());
                    record1.setUserId(entity.getUserId());
                    record1.setDirection(1);//买卖方向(0开仓 1平仓)
                    record1.setType(0);//委托类型(0市价 1限价)
                    record1.setNumber(entity.getTradableNumber());//委托手数
                    record1.setBalance(getMarketValue);//市值>>>金额
                    record1.setCreateTime(getDate);
                    int addTransferWarehouseRecor = transferWarehouseRecordDao.insert(record1);


                    //添加期权账户流水记录
                    CdBankStatementsEntity bankStatements = new CdBankStatementsEntity();
                    bankStatements.setType(2);//类型（1银行账户 2期货账户）
                    bankStatements.setCoinType("人民币");
                    bankStatements.setBeforeAmount(nowBalance);//改变前金额
                    bankStatements.setAmountMoney(getMarketValue);//这次操作的金额
                    bankStatements.setAfterAmount(newPrice);//改变后的金额
                    bankStatements.setTransferType(2);//流水类型(1:转账 2:流水)
                    bankStatements.setCreateTime(getDate);
                    bankStatements.setRemark("平仓");
                    bankStatements.setUserId(entity.getUserId());
                    int addBankStatements = bankStatementsDao.insert(bankStatements);


                    if (modify1 <= 0 && updateAsset <= 0 && addBankStatements <= 0 && addTransferWarehouseRecor <= 0) {
                        return R.error("平仓失败");
                    }
                    break;
            }
        } catch (Exception e) {
            e.printStackTrace();
            return R.error();
        }
        return R.ok();
    }
}



